import pytest
import mock

from obs.libs import user
from obs.libs import credential
from obs.libs import qos
from obs.libs import admin
from obs.api.app.controllers.api import admin as admin_client


def fake_client():
    client = mock.Mock()
    client.user.return_value = ""
    client.qos.limits.return_value = ""
    client.user.credentials.return_value = ""
    client.user.credentials.status.return_value = ""

    return client


def fake_list_users(client, group_id, user_type="all", user_status="active", limit=""):
    user1 = {
        "userId": "jerrygarcia",
        "fullName": "Jerry Garcia",
        "emailAddr": "garcia@bgn.net",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "active": "true",
        "canonicalUserId": "123",
    }
    user2 = {
        "userId": "johnthompson",
        "fullName": "John Thompson",
        "emailAddr": "",
        "address1": "",
        "city": "",
        "active": "true",
        "canonicalUserId": "123",
    }
    return [user1, user2]


def test_list(client, monkeypatch):
    monkeypatch.setattr(user, "list_user", fake_list_users)

    result = client.get("/api/admin/user", data={"groupId": "test"})
    assert result.get_json()["data"] == [
        {
            "userId": "jerrygarcia",
            "fullName": "Jerry Garcia",
            "emailAddr": "garcia@bgn.net",
            "address1": "456 Shakedown St.",
            "city": "Portsmouth",
            "active": "true",
            "canonicalUserId": "123",
        },
        {
            "userId": "johnthompson",
            "fullName": "John Thompson",
            "emailAddr": "",
            "address1": "",
            "city": "",
            "active": "true",
            "canonicalUserId": "123",
        },
    ]


def fake_qos_info(client, user_id, group_id):
    qos = {
        "groupId": "testing",
        "userId": "foo",
        "labelId": "foobar",
        "qosLimitList": [
            {"type": "STORAGE_QUOTA_KBYTES", "value": -1},
            {"type": "REQUEST_RATE_LW", "value": -1},
            {"type": "REQUEST_RATE_LH", "value": -1},
            {"type": "DATAKBYTES_IN_LW", "value": -1},
            {"type": "DATAKBYTES_IN_LH", "value": -1},
            {"type": "DATAKBYTES_OUT_LW", "value": -1},
            {"type": "DATAKBYTES_OUT_LH", "value": -1},
            {"type": "STORAGE_QUOTA_COUNT", "value": -1},
        ],
        "Storage Limit": "unlimited",
    }
    return qos


def test_qos_info(client, monkeypatch):
    monkeypatch.setattr(qos, "info", fake_qos_info)

    result = client.get("/api/admin/qos", data={"groupId": "test", "userId": "foo"})
    assert "unlimited" in result.get_json()["data"].values()


def fake_cred_list(client, user_id, group_id):
    cred = [{"accessKey": "123", "secretKey": "abc", "createDate": 000, "active": True}]
    return cred


def test_cred_list(client, monkeypatch):
    monkeypatch.setattr(credential, "list", fake_cred_list)

    result = client.get("/api/admin/cred", data={"groupId": "test", "userId": "foo"})
    assert result.get_json()["data"] == [
        {"accessKey": "123", "secretKey": "abc", "createDate": 000, "active": True}
    ]


def test_delete_user(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.delete("/api/admin/user", data={"groupId": "test", "userId": "foo"})
    assert result.get_json()["message"] == f"User foo deleted successfully."


def test_create_user(client, monkeypatch):
    pass


def test_set_qos(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.post(
        "/api/admin/qos", data={"groupId": "test", "userId": "foo", "limit": 100}
    )
    assert result.get_json()["message"] == f"User foo quota changed successfully."


def fake_usage():
    client_user = mock.Mock()
    client_user.usage.return_value = [
        {
            "groupId": "test",
            "userId": "foo",
            "region": "stage",
            "operation": "SB",
            "uri": "",
            "timestamp": "0",
            "value": "100",
            "count": "0",
            "whitelistValue": "0",
            "whitelistCount": "0",
            "maxValue": "0",
            "whitelistMaxValue": "0",
            "ip": "",
            "bucket": None,
            "policyId": None,
            "averageValue": "100",
            "whitelistAverageValue": "0",
        }
    ]
    return client_user


def test_usage(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_usage)

    result = client.get("/api/admin/usage", data={"groupId": "test", "userId": "foo"})
    assert "100" in result.get_json()["data"].values()


def test_create_cred(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.post("/api/admin/cred", data={"groupId": "test", "userId": "foo"})
    assert (
        result.get_json()["message"] == f"User foo new credential created successfully."
    )


def test_remove_cred(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.delete("/api/admin/cred", data={"access_key": "123"})
    assert result.get_json()["message"] == f"Access key 123 deleted successfully."


def test_status_cred(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.put("/api/admin/cred", data={"access_key": "123", "status": "true"})
    assert result.get_json()["message"] == f"Credential status has been activated."


def fake_user_info(client, group_id, user_id):
    user = {
        "userId": "jerrygarcia",
        "fullName": "Jerry Garcia",
        "emailAddr": "garcia@bgn.net",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "active": "true",
        "canonicalUserId": "123",
    }
    return user


def test_suspend_user(client, monkeypatch):
    monkeypatch.setattr(user, "info", fake_user_info)
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.put(
        "/api/admin/user", data={"groupId": "test", "userId": "foo", "suspend": "true"}
    )
    assert result.status_code == 200
    assert result.get_json()["message"] == "User has been suspended"


def test_create_user(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_client", fake_client)

    result = client.post(
        "/api/admin/user",
        data={
            "groupId": "test",
            "userId": "foo",
            "fullName": "foobar",
            "quotaLimit": 1000,
        },
    )
    assert result.get_json()["message"] == f"User foo created successfully."



import os
from dotenv import load_dotenv
from obs.api.app.libs import MetaFlaskEnv

APP_ROOT = os.path.join(os.path.dirname(__file__), "../..")
dotenv_path = os.path.join(APP_ROOT, ".env")
load_dotenv(dotenv_path)


class Config(MetaFlaskEnv):
    ENV_PREFIX = "FLASK_"



import os
import re
import sys
import tzlocal
import xmltodict
from datetime import datetime
from distutils.util import strtobool


def get_bucket_key(uri):
    """Return bucket name and key from given S3 URI"""
    if uri.startswith("s3://"):
        uri = uri[5:]

    components = uri.split("/")
    bucket = components[0]
    key = ""
    if len(components) > 1:
        key = "/".join(components[1:])
    return bucket, key


def sizeof_fmt(num, suffix="B"):
    """Convert any number to human readable size. By Fred Cirera"""
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return "%3.1f %s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f %s%s" % (num, "Yi", suffix)


def check(response):
    """Check if response contains error result
    then raise exception."""
    sys.tracebacklimit = 0

    if "reason" in response:
        msg = (
            f"{response.get('reason')} [{response.get('status_code')}]\n"
            f"URL: {response.get('url')}"
        )
        raise ValueError(msg)


def check_plain(response):
    """Check if response contains error result
    then raise exception."""
    if response.text:
        response_dict = xmltodict.parse(response.text)

        if "Error" in response_dict:
            code = response_dict["Error"]["Code"]
            msg = response_dict["Error"]["Message"]
            raise ValueError(f"{code}: {msg}")


def human_date(unixtime):
    """Convert unix timestamp to human readable."""
    # divided by 1000 to convert from miliseconds to seconds
    unix_timestamp = int(unixtime) / 1000
    local_timezone = tzlocal.get_localzone()
    local_time = datetime.fromtimestamp(unix_timestamp, local_timezone)
    human_datetime = local_time.strftime("%Y-%m-%d %H:%M:%S%z (%Z)")
    return human_datetime


def set_id(ids):
    """Set ID  for granting access"""
    if ids != None:
        ids = [f"id={id}" for id in ids.split(",")]
        return ",".join(ids)
    else:
        return ""


def compatibility():
    """get neo compatibility environment"""
    return strtobool(os.environ.get("OBS_USE_NEO"))


def user_sanitize(uIDs):
    """Sanitize input when creating new user"""
    uID = {}
    regex = r"[<>`;|&#]|[\\n]{2}|[%26]{3}|\n"
    regexuid = r"[^\w@#_\-.]"

    for key, value in uIDs.items():
        if key == "userId":
            uID[key] = re.sub(regexuid, "", value)
        elif type(value) == str:
            uID[key] = re.sub(regex, "", value)
        else:
            uID[key] = value
    return uID


def sanitize(type, inp):
    """Sanitize input with spesific type"""
    regex = {"bucket": r"[^a-z0-9.-]", "upload": r"[\"\{}^%`\]\[~<>|#]|[^\x00-\x7F]"}

    return re.sub(regex[type], "", inp)



import os
import boto3
from distutils.util import strtobool
from cloudianapi.client import CloudianAPIClient
from requests_aws4auth import AWS4Auth


def get_endpoint(url, bucket=None):
    """generate endpoint.

    Use `http` if ssl False otherwise `https`
    Use `example.com` if bucket False otherwise `bucketname.example.com`
    """
    hostname = {
        "storage": os.environ.get("OBS_USER_URL"),
        "admin": os.environ.get("OBS_ADMIN_URL"),
    }
    ssl = strtobool(os.environ.get("OBS_USE_HTTPS"))
    protocol = f"http{ssl and 's' or ''}://"
    bucket_name = f"{bucket and bucket or ''}{bucket and '.' or ''}"
    endpoint = f"{protocol}{bucket_name}{hostname[url]}"
    return endpoint


def resource():
    """Take credential and create boto session

    :return: resource service client.
    """
    access_key = os.environ.get("OBS_USER_ACCESS_KEY")
    secret_key = os.environ.get("OBS_USER_SECRET_KEY")
    endpoint = get_endpoint("storage")

    sess = boto3.Session(aws_access_key_id=access_key, aws_secret_access_key=secret_key)
    s3_resource = sess.resource("s3", endpoint_url=endpoint)

    return s3_resource


def plain_auth():
    """Sign S3 the auth manually"""
    access_key = os.environ.get("OBS_USER_ACCESS_KEY")
    secret_key = os.environ.get("OBS_USER_SECRET_KEY")

    # "eu-west-1" just fake region that we didn't use
    # it must be supplied otherwise key sign failed
    auth = AWS4Auth(access_key, secret_key, "eu-west-1", "s3")

    return auth


def admin_client():
    """:return: CloudianApiClient object"""
    user = os.environ.get("OBS_ADMIN_USERNAME")
    passwd = os.environ.get("OBS_ADMIN_PASSWORD")
    endpoint = get_endpoint("admin")
    port = os.environ.get("OBS_ADMIN_PORT")
    client = CloudianAPIClient(url=endpoint, user=user, key=passwd, port=port)
    return client



import click
import bitmath

from obs.libs import qos as qos_lib
from obs.libs import utils


def get_limit_kbytes(qos):
    """Get kilobytes limit of storage in QoS response."""
    qos_limits = qos["qosLimitList"]
    limit = 0
    for item in qos_limits:
        if item["type"] == "STORAGE_QUOTA_KBYTES":
            limit = item["value"]
            break

    if limit == -1:
        limit = "Unlimited"
    else:
        limit = bitmath.kB(limit).best_prefix()

    return limit


def info(client, user_id, group_id):
    try:
        qos = qos_lib.info(client, user_id=user_id, group_id=group_id)
        utils.check(qos)
        qos_limit = get_limit_kbytes(qos)
        msg = (
            f"Group ID: {qos['groupId']}\n"
            f"User ID: {qos['userId']}\n"
            f"Storage Limit: {qos_limit}"
        )
        click.secho(msg)
    except Exception as exc:
        click.secho(
            f"Storage limit fetching failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def set(client, user_id, group_id, limit):
    try:
        qos = qos_lib.set(client, user_id=user_id, group_id=group_id, limit=limit)
        utils.check(qos)
        click.secho("Storage limit changed", fg="green")
    except Exception as exc:
        click.secho(
            f"Storage limit change failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def rm(client, user_id, group_id):
    try:
        qos = qos_lib.rm(client, user_id=user_id, group_id=group_id)
        utils.check(qos)
        click.secho("Storage limit removed", fg="green")
    except Exception as exc:
        click.secho(
            f"Storage limit removal failed. \n{exc}", fg="yellow", bold=True, err=True
        )



import pytest
import mock
import os
import requests
import xmltodict
from datetime import datetime

import obs.libs.bucket
import obs.libs.auth
import obs.libs.gmt
import obs.libs.utils
import obs.libs.config
from obs.cli.main import cli
from click.testing import CliRunner


def fake_resource():
    pass


def fake_plain_auth():
    pass


def load_config_file():
    pass


@pytest.fixture
def resource(monkeypatch):
    monkeypatch.setattr(obs.libs.config, "load_config_file", load_config_file)
    monkeypatch.setattr(obs.libs.auth, "resource", fake_resource)


@pytest.fixture
def plain_auth(monkeypatch):
    monkeypatch.setattr(obs.libs.config, "load_config_file", load_config_file)
    monkeypatch.setattr(obs.libs.auth, "plain_auth", fake_plain_auth)


def fake_buckets(resource):
    bucket1 = mock.Mock()
    bucket1.name = "bucket-one"
    dt1 = datetime(2019, 9, 24, 1, 1, 0, 0)
    bucket1.creation_date = dt1

    bucket2 = mock.Mock()
    bucket2.name = "bucket-two"
    bucket2.creation_date = dt1

    return [bucket1, bucket2]


def fake_session(**kwargs):
    session = mock.Mock()
    session.resource.return_value = "s3_resource"
    return session


def test_resources(monkeypatch):
    monkeypatch.setattr(obs.libs.config, "config_file", lambda: "home/user/path")

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "ls"])
    assert result.output == (
        f"[Errno 2] No such file or directory: 'home/user/path'\n"
        f"Configuration file not available.\n"
        f"Consider running 'obs --configure' to create one\n"
    )


def test_plain_auth(monkeypatch):
    monkeypatch.setattr(obs.libs.config, "config_file", lambda: "home/user/path")

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "info", "s3://satu/"])
    assert result.output == (
        f"[Errno 2] No such file or directory: 'home/user/path'\n"
        f"Configuration file not available.\n"
        f"Consider running 'obs --configure' to create one\n"
    )


def test_ls(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "buckets", fake_buckets)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "ls"])
    assert result.output == (
        f"2019-09-24 01:01:00 bucket-one\n" f"2019-09-24 01:01:00 bucket-two\n"
    )


def fake_exc_buckets(resource):
    bucket1 = mock.Mock()
    bucket1.name = "bucket-one"
    bucket1.creation_date = "date"

    return [bucket1]


def test_except_ls(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "buckets", fake_exc_buckets)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "ls"])
    assert result.output == (f"Bucket listing failed. \n" f"Invalid format specifier\n")


def fake_get_objects(resource, bucket_name, prefix=None):
    return {
        "Contents": [
            {
                "Key": "foo.txt",
                "LastModified": datetime(2019, 9, 24, 1, 1, 0, 0),
                "ETag": '"d41d8cd98f00b204e9800998ecffake"',
                "Size": 36,
                "StorageClass": "STANDARD",
                "Owner": {
                    "DisplayName": "john doe",
                    "ID": "5ac765187f93d3f1cef810afakefake",
                },
            }
        ],
        "CommonPrefixes": [{"Prefix": "a/b/"}],
    }


def test_ls_storage(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "get_objects", fake_get_objects)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "ls", "s3://bucket-two/a/b/"])

    assert "bucket-two/foo.txt" in result.output


def test_empty_storage(monkeypatch, resource):
    monkeypatch.setattr(
        obs.libs.bucket, "get_objects", lambda resource, bucket_name, prefix: ""
    )

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "ls", "bucket-one"])

    assert result.output == (
        f'Bucket "bucket-one" is empty\n' f"string indices must be integers\n"
    )


def fake_get_files(resource, bucket_name, prefix=""):
    obj1 = mock.Mock()
    obj1.key = "ddg.png"
    obj1.size = 234

    obj2 = mock.Mock()
    obj2.key = "obj.txt"
    obj2.size = 793

    return [obj1, obj2]


def test_bucket_usage(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "get_files", fake_get_files)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "du", "s3://bucket-one/"])

    assert result.output == (f'1.00 KiB, 2 objects in "bucket-one" bucket\n')


def fake_bucket_info(resource, bucket_name, auth):
    acl = [[["Test user"], ["FULL_CONTROL"]], [["Public"], ["FULL_CONTROL"]]]
    info = {
        "ACL": acl,
        "CORS": None,
        "Policy": None,
        "Expiration": None,
        "Location": "US",
        "GmtPolicy": "2 Replication in Midplaza, 1 in Technovillage",
    }

    return info


def test_bucket_info(monkeypatch, resource, plain_auth):
    monkeypatch.setattr(obs.libs.bucket, "bucket_info", fake_bucket_info)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "info", "bucket-one"])
    assert result.output == (
        f"Location: US\n"
        f"Expiration Rule: None\n"
        f"Policy: None\n"
        f"CORS: None\n"
        f"ACL: ['Test user'] : ['FULL_CONTROL']\n"
        f"ACL: ['Public'] : ['FULL_CONTROL']\n"
        f"Gmt Policy: 2 Replication in Midplaza, 1 in Technovillage\n"
    )


def fake_exc_bucket_info(resource, bucket_name, auth):
    acl = [["Test user"], ["FULL_CONTROL"]]
    info = {
        "ACL": acl,
        "CORS": None,
        "Policy": None,
        "Expiration": None,
        "Location": "US",
        "GmtPolicy": "2 Replication in Midplaza, 1 in Technovillage",
    }
    return info


def test_except_bucket_info(resource, plain_auth, monkeypatch):
    monkeypatch.setattr(obs.libs.bucket, "bucket_info", fake_exc_bucket_info)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "info", "bucket-one"])

    assert result.output == (
        f"Location: US\n"
        f"Expiration Rule: None\n"
        f"Policy: None\n"
        f"CORS: None\n"
        f"Info fetching failed. \n"
        f"list index out of range\n"
    )


def fake_object_info(resource, bucket_name, object_name):
    acl = [[["Test user"], ["FULL_CONTROL"]]]
    dt = datetime(2019, 9, 24, 13, 18, 7, 0)
    info = {
        "ACL": acl,
        "Size": 300,
        "LastModified": dt,
        "MD5": "5610180790cf66a71cf5ea9ad0f920f5",
        "MimeType": "binary/octet-stream",
        "StorageClass": None,
    }

    return info


def test_object_info(monkeypatch, resource, plain_auth):
    monkeypatch.setattr(obs.libs.bucket, "object_info", fake_object_info)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "info", "s3://bucket-one/logo.png"])

    assert result.output == (
        f"File Size: 300.0 B\n"
        f"Last Modified: 2019-09-24 13:18:07\n"
        f"Mime Type: binary/octet-stream\n"
        f"Storage: None\n"
        "MD5 Sum: 5610180790cf66a71cf5ea9ad0f920f5\n"
        f"ACL: ['Test user'] : ['FULL_CONTROL']\n"
    )


def fake_exc_object_info(resource, bucket_name, object_name):
    info = {
        "ACL": "foo",
        "Size": 300,
        "LastModified": "foo",
        "MD5": "5610180790cf66a71cf5ea9ad0f920f5",
        "MimeType": "binary/octet-stream",
        "StorageClass": None,
    }

    return info


def test_except_object_info(resource, plain_auth, monkeypatch):
    monkeypatch.setattr(obs.libs.bucket, "object_info", fake_exc_object_info)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "info", "s3://bucket-one/logo.png"])

    assert result.output == (f"Info fetching failed. \n" f"Invalid format specifier\n")


def fake_presign(resource, bucket_name, object_name, expire):
    return "https://myendpotin.net/oneoneone/logo.png?AWSAccessKeyId=62fake123&Signature=rSNa6fake&Expires=15"


def test_presign(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "generate_url", fake_presign)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "presign", "s3://bucket-one/logo.png"])

    assert (
        result.output
        == "https://myendpotin.net/oneoneone/logo.png?AWSAccessKeyId=62fake123&Signature=rSNa6fake&Expires=15\n"
    )


def test_except_presign(monkeypatch, resource):

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "presign", "s3://bucket-one/logo.png"])

    assert result.output == (
        f"URL generation failed. \n" f"'NoneType' object has no attribute 'meta'\n"
    )


def create_fake_buck(objsize):
    buck = mock.Mock()
    buck.total_objects = len(objsize)
    buck.total_size = sum(objsize)
    return buck.total_size, buck.total_objects


def fake_disk_usage(resource):
    bucket_name = ["green", "black"]
    disk_usages = []
    buck = []
    buck.append(create_fake_buck([100, 200]))
    buck.append(create_fake_buck([400, 150, 250]))
    for bckt, obj in zip(bucket_name, buck):
        disk_usages.append([bckt, obj])
    return disk_usages


def test_disk_usage(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "disk_usage", fake_disk_usage)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "du"])

    assert result.output == (
        f'300.00 Byte, 2 objects in "green" bucket\n'
        f'800.00 Byte, 3 objects in "black" bucket\n'
        f"---\n"
        f"1.07 KiB Total\n"
    )


def fake_exc_disk_usage(resource):
    bucket_name = ["green", "black"]
    disk_usages = []
    buck = []
    buck.append(create_fake_buck(12))
    buck.append(create_fake_buck(10))
    for bckt, obj in zip(bucket_name, buck):
        disk_usages.append([bckt, obj])
    return disk_usages


def test_except_disk_usage(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.bucket, "disk_usage", fake_exc_disk_usage)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "du"])

    assert result.output == (
        f"Disk usage fetching failed. \n" f"object of type 'int' has no len()\n"
    )


def fake_gmt():
    policies = {
        "Jakarta": {"id": "123", "desc": "", "_": ""},
        "Sydney": {"id": "143", "desc": "foo", "_": ""},
    }
    return policies


def test_gmt(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.gmt, "get_policies", fake_gmt)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "gmt", "--policy-id"])

    assert result.output == (
        f"Name: Jakarta\n"
        f"Id: 123\n"
        f"Description: No description\n\n"
        f"Name: Sydney\n"
        f"Id: 143\n"
        f"Description: foo\n\n"
    )


def test_notset_gmt(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.gmt, "get_policies", lambda: "notset")

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "gmt", "--policy-id"])

    assert result.output == (
        f"Can't find Policy file\n"
        f"See '#using-cloudian-hyperstore-extension' in our documentation for more information\n"
    )


def test_except_gmt(monkeypatch, resource):
    monkeypatch.setattr(obs.libs.gmt, "get_policies", lambda: {"foo": "bar"})

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "gmt", "--policy-id"])

    assert result.output == (
        f"Show policies failed. \n" f"'str' object has no attribute 'values'\n"
    )


def fake_move():
    bucket = mock.Mock()
    bucket.Object.return_value.copy.side_effect = lambda source: None
    bucket.Object.return_value.delete.side_effect = lambda: None
    return bucket


def test_mv(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_move)
    monkeypatch.setattr(obs.libs.bucket, "is_exists", lambda res, bucket, object: True)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["storage", "mv", "s3://bucket-one/obj1", "s3://bucket-two"]
    )

    assert result.output == f'Object "obj1" moved to "bucket-two" bucket successfully\n'


def test_except_mv(monkeypatch, resource):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_move)
    monkeypatch.setattr(obs.libs.bucket, "is_exists", lambda res, bucket, object: False)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["storage", "mv", "s3://bucket-one/obj1", "s3://bucket-two"]
    )

    assert result.output == (f"Object move failed. \n" f"Object not exists: obj1\n")


def fake_copy():
    bucket = mock.Mock()
    bucket.Object.return_value.copy.side_effect = lambda source: None
    return bucket


def test_cp(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_copy)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["storage", "cp", "s3://bucket-one/obj1", "s3://bucket-two/"]
    )

    assert result.output == f'Object "obj1" copied successfully\n'


def test_except_cp(resource):
    runner = CliRunner()
    result = runner.invoke(
        cli, ["storage", "cp", "s3://bucket-one/obj1", "s3://bucket-two/"]
    )

    assert result.output == (
        f"Object copy failed. \n" f"'NoneType' object has no attribute 'Object'\n"
    )


def fake_remove():
    bucket = mock.Mock()
    bucket.Object.return_value.delete.side_effect = lambda: None
    return bucket


def test_rm_object(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_remove)
    monkeypatch.setattr(obs.libs.bucket, "is_exists", lambda res, bucket, object: True)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "rm", "s3://bucket-one/obj1"])

    assert result.output == f'Object "obj1" removed successfully\n'


def test_except_rm_object(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_remove)
    monkeypatch.setattr(obs.libs.bucket, "is_exists", lambda res, bucket, object: False)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "rm", "s3://bucket-one/obj1"])

    assert result.output == (f"Object removal failed. \n" f"Object not exists: obj1\n")


def fake_mb(*args, **kwargs):
    requests = mock.Mock()
    requests.put.side_effect = lambda: None
    return requests


def test_mb(monkeypatch, plain_auth):
    monkeypatch.setattr(obs.libs.auth, "strtobool", lambda http: True)
    monkeypatch.setattr(requests, "put", fake_mb)
    monkeypatch.setattr(obs.libs.utils, "compatibility", lambda: True)
    monkeypatch.setattr(obs.libs.utils, "check_plain", lambda response: None)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "mb", "bucket-one"])

    assert result.output == f'Bucket "bucket-one" created successfully\n'


def fake_response(**kwargs):
    response = mock.Mock()
    response.text = "Done"
    return response


def test_except_mb(monkeypatch, plain_auth, resource):
    monkeypatch.setattr(obs.libs.bucket, "neo_create_bucket", fake_response)
    monkeypatch.setattr(obs.libs.utils, "compatibility", lambda: True)
    monkeypatch.setattr(
        xmltodict,
        "parse",
        lambda response: {"Error": {"Code": "101", "Message": "Error"}},
    )

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "mb", "bucket-one"])

    assert result.output == (f"Bucket creation failed. \n" f"101: Error\n")


def fake_remove_bucket():
    bucket = mock.Mock()
    bucket.Bucket.return_value.delete.side_effect = lambda: None
    return bucket


def test_rm_bucket(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_remove_bucket)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "rm", "s3://bucket-one/"])

    assert result.output == f'Bucket "bucket-one" deleted successfully.\n'


def test_except_rm_bucket(monkeypatch, resource):
    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "rm", "bucket-one"])

    assert result.output == f"'NoneType' object has no attribute 'Bucket'\n"


def fake_acl_object():
    acl = mock.Mock()
    acl.info = []
    acl.Object.return_value.Acl.return_value.put.side_effect = acl.info.append(
        [["Testing"], ["FULL_CONTROL"]]
    )
    return acl


def fake_acl_Bucket():
    acl = mock.Mock()
    acl.info = [[["Test user"], ["FULL_CONTROL"]]]
    acl.Bucket.return_value.Acl.return_value.put.side_effect = acl.info.append(
        [["Testing"], ["FULL_CONTROL"]]
    )
    return acl


def test_acl_bucket(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_acl_Bucket)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "acl", "bucket-one", "private"])

    assert fake_acl_Bucket().info == [
        [["Test user"], ["FULL_CONTROL"]],
        [["Testing"], ["FULL_CONTROL"]],
    ]
    assert result.output == f"ACL changed successfully\n"


def test_acl_object(monkeypatch):
    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", fake_acl_object)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "acl", "s3://bucket-one/obj1", "private"])
    assert fake_acl_object().info == [[["Testing"], ["FULL_CONTROL"]]]

    assert result.output == f"ACL changed successfully\n"


def test_except_acl(resource):
    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "acl", "bucket-one", "private"])

    assert result.output == (
        f"ACL change failed. \n" f"'NoneType' object has no attribute 'Bucket'\n"
    )


def test_get(monkeypatch, fs, resource):
    def donwload():
        fs.create_file("/obj1.jpg")
        resource = mock.Mock()
        resource.Object.return_value.download_file.side_effect = lambda name: None
        return resource

    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", donwload)
    monkeypatch.setattr(
        obs.libs.bucket,
        "list_download",
        lambda bucket_name, prefix, list_objects: "obj1.png",
    )
    monkeypatch.setattr(
        obs.libs.bucket, "is_exists", lambda resource, bucket, object: True
    )

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "get", "s3://bucket-one/obj1.jpg"])

    assert os.path.exists("/obj1.jpg")
    assert result.output == f'Object "obj1.jpg" downloaded successfully\n'


def test_except_get(monkeypatch, resource):
    monkeypatch.setattr(
        obs.libs.bucket,
        "list_download",
        lambda bucket_name, prefix, list_objects: ["obj1.png"],
    )
    monkeypatch.setattr(
        obs.libs.bucket, "is_exists", lambda resource, bucket, object: False
    )

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "get", "s3://bucket-one/obj1.png"])

    assert result.output == (f"Object download failed. \nObject not exists: obj1.png\n")


def test_put(monkeypatch, fs, resource):
    def upload(**kwargs):
        fs.create_file("upload/obj1.png")

    monkeypatch.setattr(obs.libs.bucket, "upload_object", upload)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["storage", "put", "obj1.png", "s3://bucket-one/obj1.png"]
    )

    assert os.path.exists("upload/obj1.png")
    assert result.output == f'Object "obj1.png" uploaded successfully\n'


def fake_dir():
    resource = mock.Mock()
    resource.meta.client.put_object.side_effect = lambda **kwargs: "done"
    return resource


def test_mkdir(monkeypatch, fs):
    def mkdir():
        fs.create_dir("/new/")
        resource = mock.Mock()
        resource.meta.client.put_object.side_effect = lambda **kwargs: None
        return resource

    monkeypatch.setattr(obs.cli.storage.commands, "get_resources", mkdir)

    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "mkdir", "s3://bucket-one/obs"])

    assert os.path.exists("/new/")
    assert result.output == f'Directory "obs" created successfully\n'


def test_except_mkdir(resource):
    runner = CliRunner()
    result = runner.invoke(cli, ["storage", "mkdir", "s3://bucket-one/obs"])
    assert result.output == (
        f"Directory creation failed. \n" f"'NoneType' object has no attribute 'meta'\n"
    )



from flask import Response
import json


def response(status_code, message=None, data=None):
    """Response data helper

    Arguments:
        status_code {int} -- http status code

    Keyword Arguments:
        message {string} -- response message (default: {None})
        data {dict} -- data to be appended to response (default: {None})

    Returns:
        dict -- response data
    """
    success_status = {
        200: "Operation succeeded",
        201: "Created",
        202: "Accepted",
        204: "Reply does not contain additional content",
        304: "Not modified",
    }

    failure_status = {
        400: "Internal error occurred - unexpected error caused by request data",
        401: "Unauthorized operation",
        403: "Forbidden operation",
        404: "Specified object not found",
        405: "Method Not Allowed, for example, resource doesn't support DELETE method",
        406: "Method Not Acceptable",
        409: "Conflict",
        423: "Locked",
        426: "Upgrade Required",
        500: "Internal Server Error - unexpected server-side error",
        501: "Not Implemented - functionality is not implemented on the server side",
        503: "Service is unavailable",
    }

    status = {}
    status["code"] = status_code

    if status_code in success_status:
        status["count"] = len(data) if data else 0
        status["data"] = data if data else None
        status["status"] = "success"
        status["message"] = message if message else success_status[status_code]
    elif status_code in failure_status:
        status["status"] = "error"
        status["message"] = message if message else failure_status[status_code]
    else:
        status["status"] = "error"
        status["message"] = message if message else failure_status[400]

    response = Response(
        response=json.dumps(status), status=status_code, mimetype="application/json"
    )

    return response



import mock
import pytest
from obs.libs import qos


@pytest.fixture
def limit():
    return {
        "groupId": "testing",
        "userId": "StageTest",
        "labelId": "qos.userQosOverrides.title",
        "qosLimitList": [
            {"type": "STORAGE_QUOTA_KBYTES", "value": -1},
            {"type": "REQUEST_RATE_LW", "value": -1},
            {"type": "REQUEST_RATE_LH", "value": -1},
            {"type": "DATAKBYTES_IN_LW", "value": -1},
            {"type": "DATAKBYTES_IN_LH", "value": -1},
            {"type": "DATAKBYTES_OUT_LW", "value": -1},
            {"type": "DATAKBYTES_OUT_LH", "value": -1},
            {"type": "STORAGE_QUOTA_COUNT", "value": -1},
        ],
    }


def fake_client():
    client = mock.Mock()
    client.qos.limits.return_value = limit()
    return client


def test_info():
    assert qos.info(fake_client(), "StageTest", "testing") == limit()


def test_set():
    assert qos.set(fake_client(), "user", "group", "limit") == limit()


def test_rm():
    assert qos.rm(fake_client(), "user", "group") == limit()



def list(client, user_id, group_id):
    """Get security credentials of  user."""
    credentials = client.user.credentials.list(
        method="GET", userId=user_id, groupId=group_id
    )
    return credentials


def status(client, access_key, status=True):
    """Set security credentials status."""
    status = client.user.credentials.status(
        method="POST", accessKey=access_key, isActive=status
    )
    return status


def rm(client, access_key):
    """Remove security credentials."""
    credentials = client.user.credentials(method="DELETE", accessKey=access_key)
    return credentials


def create(client, user_id, group_id):
    """Create security credentials."""
    credentials = client.user.credentials(
        method="PUT", userId=user_id, groupId=group_id
    )
    return credentials



import pytest
import mock
import uuid
from obs.libs import gmt
from obs.libs import bucket


@pytest.fixture
def resource(self):
    pass


class Testbucket:
    def fake_resource(self):
        resource = mock.Mock()
        resource.buckets.all.return_value = ["bucket-one", "satu-b21f6bb7-39d5"]
        resource.Bucket.return_value.objects.filter.return_value = [
            "bucket-one",
            "obj.png",
        ]
        return resource

    def fake_resource_(self):
        resource = mock.Mock()
        resource.buckets.all.return_value = ["bucket-one", "satu-b21f6bb7-39d5"]
        resource.Bucket.return_value.objects.filter.return_value = [
            "bucket-one",
            "obj.png",
        ]
        return resource

    def test_buckets(self):
        assert bucket.buckets(self.fake_resource()) == [
            "bucket-one",
            "satu-b21f6bb7-39d5",
        ]

    def test_random(self, monkeypatch):
        monkeypatch.setattr(uuid, "uuid4", lambda: "71e43e94-c10d")
        assert bucket.gen_random_name("awesome") == f"awesome-71e43e94-c10d"

    def fake_get_files(self, resource, bucket_nam, prefix=None):
        obj1 = mock.Mock()
        obj1.key = "ddg.png"
        obj1.size = 234

        obj2 = mock.Mock()
        obj2.key = "obj.txt"
        obj2.size = 793

        return [obj1, obj2]

    def fake_get_objects(self, resource, bucket_nam, prefix=None):
        return {
            "Contents": [{"Key": "ddg.png"}],
            "CommonPrefixes": [{"Prefix": "a/b/"}],
        }

    def test_exists(self, monkeypatch):
        monkeypatch.setattr(bucket, "get_objects", self.fake_get_objects)

        assert bucket.is_exists(resource, "satu", "ddg.png") is True
        assert bucket.is_exists(resource, "satu", "ang.png") is False

    def test_bucket_usage(self, monkeypatch):
        monkeypatch.setattr(bucket, "get_files", self.fake_get_files)
        assert bucket.bucket_usage(resource, "bucket-name") == (1027, 2)

    def fake_bucket(self, resource):
        buckets = []
        for count in range(3):
            bucket = mock.Mock()
            bucket.name = f"bucket{count}"
            buckets.append(bucket)
        return buckets

    def fake_usage(self, resource, name):
        bucket = mock.Mock()
        # number 3 is used to add bucket number with 3
        # number 273 is used to multiply bucket size with 273
        bucket.objects = int(name[-1]) + 3
        bucket.size = bucket.objects * 273
        return bucket.size, bucket.objects

    def test_disk_usage(self, monkeypatch):
        monkeypatch.setattr(bucket, "buckets", self.fake_bucket)
        monkeypatch.setattr(bucket, "bucket_usage", self.fake_usage)
        assert bucket.disk_usage("boom") == [
            ["bucket0", (819, 3)],
            ["bucket1", (1092, 4)],
            ["bucket2", (1365, 5)],
        ]

    def fake_cors(self):
        fake = mock.Mock()
        fake.Cors.return_value.cors_rules = "1jfe"
        return fake

    def test_cors(self):
        assert bucket.get_cors(self.fake_cors()) == "1jfe"

        assert bucket.get_cors("try") is None

    def fake_policy(self):
        fake = mock.Mock()
        fake.Policy.return_value.policy = "newsk"
        return fake

    def test_policy(self):
        assert bucket.get_policy(self.fake_policy()) == "newsk"

        assert bucket.get_policy("try") is None

    def fake_location(self):
        locations = mock.Mock()
        locations.get_bucket_location.return_value = {"LocationConstraint": "Jakarta"}
        return locations

    def test_location(self):
        assert bucket.get_location(self.fake_location(), "bucket-name") == "Jakarta"

    def fake_expire(self):
        fake = mock.Mock()
        fake.get_bucket_lifecycle.return_value = "Expire"
        return fake

    def test_expiration(self):
        assert bucket.get_expiration(self.fake_expire(), "bucket-name") == "Expire"

        assert bucket.get_expiration("try", "bucket-name") is None

    def fake_grant_name(self, types):
        grant = {
            "Grantee": {
                "Type": types,
                "URI": "home/grant/uri/red-deer",
                "DisplayName": "black-deer",
            },
            "Permission": "FULL CONTROL",
        }
        return grant

    def test_grant_name(self):
        assert bucket.get_grant_name(self.fake_grant_name("Group")) == "red-deer"

        assert (
            bucket.get_grant_name(self.fake_grant_name("CanonicalUser")) == "black-deer"
        )

    def fake_grants(self):
        obj = mock.Mock()
        obj.Acl.return_value.grants = [
            self.fake_grant_name("Group"),
            self.fake_grant_name("CanonicalUser"),
        ]
        return obj

    def test_grant(self):
        assert bucket.get_grants(self.fake_grants()) == [
            ["red-deer", "FULL CONTROL"],
            ["black-deer", "FULL CONTROL"],
        ]

    def test_gmt_policy(self, monkeypatch):
        monkeypatch.setattr(gmt, "policy_id", lambda name, auth: None)
        monkeypatch.setattr(
            gmt, "policy_description", lambda id: "Replica Data to all nodes"
        )
        assert (
            bucket.bucket_gmt_policy("bucket-name", "auth")
            == "Replica Data to all nodes"
        )

    def fake_client(self):
        client = mock.Mock()
        client.Bucket.return_value.meta.client = None
        return client

    def test_bucket_info(self, monkeypatch):
        monkeypatch.setattr(
            bucket, "bucket_gmt_policy", lambda name, auth: "Replica Data to all nodes"
        )
        monkeypatch.setattr(bucket, "get_grants", lambda bucket: "123")
        monkeypatch.setattr(bucket, "get_cors", lambda bucket: "None")
        monkeypatch.setattr(bucket, "get_policy", lambda bucket: "None")
        monkeypatch.setattr(bucket, "get_expiration", lambda client, bucket: "None")
        monkeypatch.setattr(bucket, "get_location", lambda client, bucket: "Jakarta")

        assert bucket.bucket_info(self.fake_client(), "bucket-name", "auth") == {
            "ACL": "123",
            "CORS": "None",
            "Policy": "None",
            "Expiration": "None",
            "Location": "Jakarta",
            "GmtPolicy": "Replica Data to all nodes",
        }

    def fake_object_resource(self):
        obj = mock.Mock()
        obj.Object.return_value.storage_class = "Group"
        obj.Object.return_value.content_type = "image/png"
        obj.Object.return_value.content_length = 1280213123
        obj.Object.return_value.last_modified = "2019-09-24 01:01:00"
        obj.Object.return_value.e_tag = "967bd45c88a91f829755742752fe9932"
        return obj

    def test_object_info(self, monkeypatch):
        monkeypatch.setattr(
            bucket, "get_grants", lambda obj: ["red-deer", "FULL CONTROL"]
        )
        assert bucket.object_info(
            self.fake_object_resource(), "bucket-name", "obj-name"
        ) == {
            "ACL": ["red-deer", "FULL CONTROL"],
            "Size": 1280213123,
            "LastModified": "2019-09-24 01:01:00",
            "MD5": "967bd45c88a91f829755742752fe9932",
            "MimeType": "image/png",
            "StorageClass": "Group",
        }

    def fake_url(self):
        client = mock.Mock()
        client.meta.client.generate_presigned_url.return_value = "https://bucket.net"
        return client

    def test_generate_url(self, monkeypatch):
        assert (
            bucket.generate_url(self.fake_url(), "bucket-name", "ddg.png")
            == "https://bucket.net"
        )



from flask import Blueprint
from flask_restful import Api
from .storage import *
from .admin import *

api_blueprint = Blueprint("api", __name__, url_prefix="/api")
api = Api(api_blueprint)

api.add_resource(bucket_api, "/storage/bucket/<bucket_name>")
api.add_resource(object_api, "/storage/object/<bucket_name>")
api.add_resource(download_object, "/storage/object/download/<bucket_name>")
api.add_resource(upload_object, "/storage/object/upload/<bucket_name>")
api.add_resource(move_object, "/storage/object/move/<bucket_name>")
api.add_resource(copy_object, "/storage/object/copy/<bucket_name>")
api.add_resource(presign, "/storage/presign/<bucket_name>/<object_name>")
api.add_resource(mkdir, "/storage/mkdir/<bucket_name>")
api.add_resource(acl, "/storage/acl")
api.add_resource(list, "/storage/list")
api.add_resource(usage, "/storage/usage")
api.add_resource(gmt_policy, "/storage/gmt")

api.add_resource(user_api, "/admin/user")
api.add_resource(qos_api, "/admin/qos")
api.add_resource(cred_api, "/admin/cred")
api.add_resource(user_usage, "/admin/usage")



import requests
import pytest
import mock
import tempfile
import os

from datetime import datetime
from obs.libs import bucket
from werkzeug.datastructures import FileStorage
from obs.api.app.controllers.api import storage


def fake_resource(access_key, secret_key):
    resouce = mock.Mock()
    resouce.Bucket.return_value.delete.side_effect = lambda: {
        "ResponseMetaData": {"RequestId": "e12"}
    }
    resouce.Object.return_value.delete.side_effect = lambda: {
        "ResponseMetaData": {"RequestId": "e12"}
    }
    resouce.Object.return_value.download_file.side_effect = lambda: ""
    resouce.Object.return_value.upload_fileobj.side_effect = (
        lambda Fileobj, ExtraArgs: ""
    )
    resouce.Object.return_value.copy.side_effect = lambda source: ""
    return resouce


def fake_buckets(resource):
    bucket1 = mock.Mock()
    bucket1.name = "bucket-one"
    dt1 = datetime(2019, 9, 24, 1, 1, 0, 0)
    bucket1.creation_date = dt1

    bucket2 = mock.Mock()
    bucket2.name = "bucket-two"
    bucket2.creation_date = dt1

    return [bucket1, bucket2]


def test_list(client, monkeypatch):
    monkeypatch.setattr(bucket, "buckets", fake_buckets)

    result = client.get(
        "/api/storage/list", data={"access_key": "123", "secret_key": "123"}
    )
    assert result.get_json()["data"] == [
        {"name": "bucket-one", "creation_date": "2019-09-24 01:01:00"},
        {"name": "bucket-two", "creation_date": "2019-09-24 01:01:00"},
    ]


def fake_list_objects(resource, bucket_name, prefix=""):
    content = {
        "Contents": [
            {
                "Key": "foo.txt",
                "LastModified": datetime(2019, 9, 24, 1, 1, 0, 0),
                "ETag": '"d41d8cd98f00b204e9800998ecffake"',
                "Size": 36,
                "StorageClass": "STANDARD",
            }
        ],
        "CommonPrefixes": [{"Prefix": "a/b/"}],
    }
    return content


def test_list_object(client, monkeypatch):
    monkeypatch.setattr(bucket, "get_objects", fake_list_objects)

    result = client.get(
        "api/storage/list",
        data={"bucket_name": "test", "access_key": "123", "secret_key": "123"},
    )
    assert {"directory": "a/b/"} in result.get_json()["data"]


def test_remove_bucket(client, monkeypatch):
    monkeypatch.setattr(storage, "get_resources", fake_resource)

    result = client.delete(
        "/api/storage/bucket/name", data={"access_key": "123", "secret_key": "123"}
    )
    assert "RequestId" in result.get_json()["data"]["ResponseMetaData"]
    assert result.get_json()["message"] == f"Bucket name deleted successfully."


def test_remove_object(client, monkeypatch):
    monkeypatch.setattr(storage, "get_resources", fake_resource)
    monkeypatch.setattr(bucket, "is_exists", lambda res, bucket, object: True)

    result = client.delete(
        "/api/storage/object/name",
        data={"access_key": "123", "secret_key": "123", "object_name": "object.png"},
    )
    assert "RequestId" in result.get_json()["data"]["ResponseMetaData"]
    assert result.get_json()["message"] == f"Object object.png deleted successfully."


def test_download(client, monkeypatch, fs):
    def download(access_key, secret_key):
        fs.create_file("obj1.jpg")
        resource = mock.Mock()
        resource.Object.return_value.download_file.side_effect = lambda name: None
        return resource

    monkeypatch.setattr(storage, "get_resources", download)
    monkeypatch.setattr(bucket, "is_exists", lambda res, bucket, object: True)

    result = client.get(
        "/api/storage/object/download/tes_bucket",
        data={"access_key": "123", "secret_key": "123", "object_name": "obj1.jpg"},
    )

    assert "obj1.jpg" in result.headers["Content-Disposition"]
    assert result.status_code == 200


def test_upload(client, monkeypatch, fs):
    fs.create_file("upload/obj1.png")
    file = FileStorage(
        filename="obj1.png",
        content_type="image/png",
        stream=open("upload/obj1.png", "rb"),
    )

    monkeypatch.setattr(storage, "get_resources", fake_resource)

    result = client.post(
        "/api/storage/object/upload/name",
        data={
            "access_key": "123",
            "secret_key": "123",
            "object_name": "/folder/obj1.jpg",
            "acl": "public",
            "files": file,
        },
        content_type="multipart/form-data",
    )
    assert "obj1.png" in os.listdir("/upload")
    assert (
        result.get_json()["message"]
        == f"Object /folder/obj1.jpg uploaded successfully."
    )


def fake_acl(access_key, secret_key):
    acl = mock.Mock()
    acl.info = [[["Test user"], ["FULL_CONTROL"]]]
    acl.Bucket.return_value.Acl.return_value.put.return_value = {
        "ResponseMetaData": {"RequestId": "e12"}
    }
    return acl


def test_acl(client, monkeypatch):
    monkeypatch.setattr(storage, "get_resources", fake_acl)

    result = client.post(
        "/api/storage/acl",
        data={"access_key": "123", "secret_key": "123", "bucket_name": "foo"},
    )
    assert "RequestId" in result.get_json()["data"]["ResponseMetaData"]


def test_mkdir(client, monkeypatch, fs):
    def mkdir(client, monkeypatch):
        fs.create_dir("new")
        resource = mock.Mock()
        resource.meta.client.put_object.return_value = {
            "ResponseMetaData": {"RequestId": "e12"}
        }
        return resource

    monkeypatch.setattr(storage, "get_resources", mkdir)

    result = client.post(
        "/api/storage/mkdir/bucket_name",
        data={"access_key": "123", "secret_key": "123", "directory": "foo"},
    )
    assert "new" in os.listdir("/")
    assert "RequestId" in result.get_json()["data"]["ResponseMetaData"]
    assert result.get_json()["message"] == f"Directory foo added successfully."


def fake_create_bucket(*args, **kwargs):
    request = mock.Mock()
    request.text = None
    return request


def test_create_bucket(client, monkeypatch):
    monkeypatch.setattr(requests, "put", fake_create_bucket)

    result = client.post(
        "/api/storage/bucket/bucket-name",
        data={"access_key": "123", "secret_key": "123"},
    )
    assert result.get_json()["message"] == f"Bucket created successfully."


def test_copy(client, monkeypatch):
    monkeypatch.setattr(storage, "get_resources", fake_resource)

    result = client.post(
        "/api/storage/object/copy/bucket_name",
        data={
            "access_key": "123",
            "secret_key": "123",
            "object_name": "obj.png",
            "copy_to": "bucket2",
        },
    )
    assert result.get_json()["message"] == f"Object obj.png copied successfully."



import click
import bitmath

from obs.libs import admin as admin_lib
from obs.libs import utils


def du(client, user_id, group_id):
    try:
        usage = admin_lib.usage(client, user_id, group_id)
        utils.check(usage)
        # earliest result must be first element (0)
        value = usage[0]["value"]
        prefixed_value = bitmath.Byte(int(value)).best_prefix()
        click.secho(f"Storage usage: {prefixed_value}")
    except Exception as exc:
        click.secho(
            f"Storage fetching failed. \n{exc}", fg="yellow", bold=True, err=True
        )



import click

from obs.libs import gmt


def show_policies():
    policies = gmt.get_policies()
    if policies == "notset":
        docs_url = "#using-cloudian-hyperstore-extension"
        msg = (
            f"Can't find Policy file\n"
            f"See '{docs_url}' in our documentation for more information"
        )
        click.secho(f"{msg}", fg="yellow", bold=True, err=True)
        return

    try:
        for zone in policies:
            policy_id, description, _ = policies[zone].values()
            if not description:
                description = "No description"
            msg = f"Name: {zone}\n" f"Id: {policy_id}\n" f"Description: {description}\n"
            click.secho(msg)
    except Exception as exc:
        click.secho(f"Show policies failed. \n{exc}", fg="yellow", bold=True, err=True)



import re
import os
import click
import bitmath

from obs.libs import bucket as bucket_lib
from obs.libs import utils


def buckets(resource):
    """Print all bucket with specified attribute."""
    try:
        all_buckets = bucket_lib.buckets(resource)
        for bucket in all_buckets:
            click.secho(f"{bucket.creation_date:%Y-%m-%d %H:%M:%S} {bucket.name}")
    except Exception as exc:
        click.secho(f"Bucket listing failed. \n{exc}", fg="yellow", bold=True, err=True)


def create_bucket(**kwargs):
    try:
        name = utils.sanitize("bucket", kwargs.pop("bucket_name"))
        if not 2 < len(name) < 64:
            raise ValueError(f"'{name}' too short or too long for bucket name")

        if utils.compatibility():
            response = bucket_lib.neo_create_bucket(**kwargs, bucket_name=name)
            utils.check_plain(response)
        else:
            response = bucket_lib.create_bucket(**kwargs, bucket_name=name)
        click.secho(f'Bucket "{name}" created successfully', fg="green")
    except Exception as exc:
        click.secho(
            f"Bucket creation failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def remove_bucket(resource, bucket_name):
    try:
        bucket_lib.remove_bucket(resource, bucket_name)
        click.secho(f'Bucket "{bucket_name}" deleted successfully.', fg="green")
    except Exception as exc:
        click.secho(f"{exc}", fg="yellow", bold=True, err=True)


def get_objects(resource, uri):
    try:
        bucket_name, prefix = utils.get_bucket_key(uri)
        response = bucket_lib.get_objects(resource, bucket_name, prefix)
        if not response:
            click.secho(f'Bucket "{bucket_name}" is empty', fg="green")

        if response["CommonPrefixes"]:
            for prefix in response["CommonPrefixes"]:
                dir_ = "DIR".rjust(12)
                click.secho(f"{dir_} {bucket_name}/{prefix['Prefix']}")

        if response["Contents"]:
            for content in response["Contents"]:
                key = content["Key"]
                size = utils.sizeof_fmt(content["Size"])
                last_modified = content["LastModified"]
                click.secho(
                    f"{last_modified:%Y-%m-%d %H:%M:%S}, {size}, {bucket_name}/{key}"
                )

    except Exception as exc:
        click.secho(f"{exc}", fg="yellow", bold=True, err=True)


def remove_object(resource, bucket_name, object_name):
    try:
        bucket_lib.remove_object(resource, bucket_name, object_name)
        click.secho(f'Object "{object_name}" removed successfully', fg="green")
    except Exception as exc:
        click.secho(f"Object removal failed. \n{exc}", fg="yellow", bold=True, err=True)


def download_object(resource, bucket_name, object_name):
    try:
        objects = bucket_lib.list_download(resource, bucket_name, object_name)
        if object_name == "":
            os.makedirs(bucket_name, exist_ok=True)
            os.chdir(bucket_name)
            name = f'Bucket "{bucket_name}"'
        elif object_name[-1] == "/":
            name = f'Directory "{object_name}"'
        else:
            name = f'Object "{object_name}"'
            objects = [object_name]
        for obj in objects:
            bucket_lib.download_object(resource, bucket_name, obj)
        click.secho(f"{name} downloaded successfully", fg="green")
    except Exception as exc:
        click.secho(
            f"Object download failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def upload_object(**kwargs):
    filename = kwargs.get("local_path")
    try:
        name = utils.sanitize("upload", kwargs.pop("object_name"))
        bucket_lib.upload_object(**kwargs, object_name=name)
        click.secho(f'Object "{filename}" uploaded successfully', fg="green")
    except Exception as exc:
        click.secho(
            f'Object "{filename}" upload failed. \n{exc}',
            fg="yellow",
            bold=True,
            err=True,
        )


def copy_object(resource, src_bucket, src_object_name, dest_bucket, dest_object_name):
    if src_object_name.endswith("/") or not src_object_name:
        click.secho(f"Object copy failed. \nExpecting filename", fg="yellow", bold=True)
        return

    try:
        bucket_lib.copy_object(
            resource, src_bucket, src_object_name, dest_bucket, dest_object_name
        )
        click.secho(f'Object "{src_object_name}" copied successfully', fg="green")
    except Exception as exc:
        click.secho(f"Object copy failed. \n{exc}", fg="yellow", bold=True, err=True)


def bucket_usage(resource, bucket_name):
    try:
        total_size, total_objects = bucket_lib.bucket_usage(resource, bucket_name)
        human_total_size = bitmath.Byte(total_size).best_prefix()
        click.secho(
            f'{human_total_size.format("{value:.2f} {unit}")}, {total_objects} objects in "{bucket_name}" bucket'
        )
    except Exception as exc:
        click.secho(
            f"Bucket usage fetching failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def disk_usage(resource):
    try:
        disk_usages = bucket_lib.disk_usage(resource)
        total_usage = 0
        for usage in disk_usages:
            bucket_name = usage[0]
            total_size, total_objects = usage[1]
            human_total_size = bitmath.Byte(total_size).best_prefix()
            total_usage += total_size
            click.secho(
                f'{human_total_size.format("{value:.2f} {unit}")}, {total_objects} objects in "{bucket_name}" bucket'
            )

        human_total_usage = bitmath.Byte(total_usage).best_prefix()
        click.secho(f"---\n" f"{human_total_usage.format('{value:.2f} {unit}')} Total")

    except Exception as exc:
        click.secho(
            f"Disk usage fetching failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def move_object(resource, src_bucket, src_object_name, dest_bucket, dest_object_name):
    if src_object_name.endswith("/") or not src_object_name:
        click.secho(f"Object move failed. \nExpecting filename", fg="yellow", bold=True)
        return

    try:
        bucket_lib.move_object(
            resource, src_bucket, src_object_name, dest_bucket, dest_object_name
        )

        click.secho(
            f'Object "{src_object_name}" moved to "{dest_bucket}" bucket successfully',
            fg="green",
        )
    except Exception as exc:
        click.secho(f"Object move failed. \n{exc}", fg="yellow", bold=True, err=True)


def bucket_info(resource, bucket_name, auth):
    try:
        info = bucket_lib.bucket_info(resource, bucket_name, auth)
        msg = (
            f"Location: {info['Location']}\n"
            f"Expiration Rule: {info['Expiration']}\n"
            f"Policy: {info['Policy']}\n"
            f"CORS: {info['CORS']}"
        )
        click.secho(msg)

        for grant in info["ACL"]:
            click.secho(f"ACL: {grant[0]} : {grant[1]}")

        if "GmtPolicy" in info:
            click.secho(f"Gmt Policy: {info['GmtPolicy']}")

    except Exception as exc:
        click.secho(f"Info fetching failed. \n{exc}", fg="yellow", bold=True, err=True)


def object_info(resource, bucket_name, object_name):
    try:
        info = bucket_lib.object_info(resource, bucket_name, object_name)
        size = utils.sizeof_fmt(info["Size"])
        last_modified = f"{info['LastModified']:%Y-%m-%d %H:%M:%S}"

        msg = (
            f"File Size: {size}\n"
            f"Last Modified: {last_modified}\n"
            f"Mime Type: {info['MimeType']}\n"
            f"Storage: {info['StorageClass']}\n"
            f"MD5 Sum: {info['MD5']}"
        )
        click.secho(msg)
        for grant in info["ACL"]:
            click.secho(f"ACL: {grant[0]} : {grant[1]}")
    except Exception as exc:
        click.secho(f"Info fetching failed. \n{exc}", fg="yellow", bold=True, err=True)


def set_acl(**kwargs):
    try:
        bucket_lib.set_acl(**kwargs)
        click.secho(f"ACL changed successfully", fg="green")
    except Exception as exc:
        click.secho(f"ACL change failed. \n{exc}", fg="yellow", bold=True, err=True)


def generate_url(resource, bucket_name, object_name, expire):
    try:
        url = bucket_lib.generate_url(resource, bucket_name, object_name, expire)
        click.secho(f"{url}")
    except Exception as exc:
        click.secho(f"URL generation failed. \n{exc}", fg="yellow", bold=True, err=True)


def mkdir(resource, bucket_name, dir_name):
    try:
        bucket_lib.mkdir(resource, bucket_name, dir_name)
        click.secho(f'Directory "{dir_name}" created successfully', fg="green")
    except Exception as exc:
        click.secho(
            f"Directory creation failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def list_multipart_upload(resource, bucket_name, prefix):
    try:
        response = bucket_lib.list_multipart_upload(resource, bucket_name, prefix)

        if response["CommonPrefixes"]:
            for prefix in response["CommonPrefixes"]:
                dir_ = "DIR".rjust(12)
                click.secho(f"{dir_} {bucket_name}/{prefix['Prefix']}")

        if response["Uploads"]:
            for content in response["Uploads"]:
                key = content["Key"]
                uid = content["UploadId"]
                last_modified = content["Initiated"]
                click.secho(
                    f"{last_modified:%Y-%m-%d %H:%M:%S}, {uid}, {bucket_name}/{key}"
                )

    except Exception as exc:
        click.secho(f"{exc}", fg="yellow", bold=True, err=True)


def list_part(resource, bucket_name, object_name, upload_id):
    try:
        response = bucket_lib.list_part(resource, bucket_name, object_name, upload_id)

        for part in response:
            size = utils.sizeof_fmt(part["Size"])
            last_modified = f"{part['LastModified']:%Y-%m-%d %H:%M:%S}"
            msg = (
                f'Number of part: {part["PartNumber"]}\n'
                f"Last Modified: {last_modified}\n"
                f'ETag: {part["ETag"]}\n'
                f"Size: {size}"
            )
            click.secho(msg)

    except Exception as exc:
        click.secho(f"{exc}", fg="yellow", bold=True, err=True)


def abort_multipart_upload(resource, bucket_name, object_name, upload_id):
    try:
        bucket_lib.abort_multipart_upload(resource, bucket_name, object_name, upload_id)
        click.secho(
            f'Multipart Upload of "{object_name}" aborted successfully', fg="green"
        )
    except Exception as exc:
        click.secho(
            f'Aborting multipart upload of "{object_name}" failed.\n{exc}',
            fg="yellow",
            bold=True,
            err=True,
        )


def complete_multipart_upload(resource, bucket_name, object_name, upload_id):
    try:
        bucket_lib.complete_multipart_upload(
            resource, bucket_name, object_name, upload_id
        )
        click.secho(
            f'Multipart Upload of "{object_name}" completed successfully', fg="green"
        )
    except Exception as exc:
        click.secho(
            f'Completing multipart upload of "{object_name}" failed.\n{exc}',
            fg="yellow",
            bold=True,
            err=True,
        )



import mock
from obs.libs import credential as cred


def fake_client():
    client = mock.Mock()
    client.user.credentials.list.return_value = {
        "accessKey": "394b287c9efake",
        "secretKey": "IgP23gfnbrguu21YqFRw4+7Mfake",
        "createDate": "1970-01-19 10:55:05+0700 (WIB)",
        "active": True,
    }
    client.user.credentials.status.return_value = "Done"
    client.user.credentials.return_value = "Done"
    return client


def test_list():
    assert cred.list(fake_client(), "user", "group") == {
        "accessKey": "394b287c9efake",
        "secretKey": "IgP23gfnbrguu21YqFRw4+7Mfake",
        "createDate": "1970-01-19 10:55:05+0700 (WIB)",
        "active": True,
    }


def test_status():
    assert cred.status(fake_client(), "key") == "Done"


def test_rm():
    assert cred.rm(fake_client(), "key") == "Done"


def test_create():
    assert cred.create(fake_client(), "user", "group") == "Done"



import uuid
import os
import requests
import logging
import zipfile

from obs.libs import gmt
from obs.libs import utils
from obs.libs import auth as auth_lib


def buckets(resource):
    """Return all available buckets object."""
    all_buckets = []
    for bucket in resource.buckets.all():
        all_buckets.append(bucket)
    return all_buckets


def gen_random_name(prefix):
    """Take random UUID and append specified prefix."""
    return f"{prefix}-{str(uuid.uuid4())[:13]}"


def create_bucket(**kwargs):
    """Create a bucket."""
    resource = kwargs.get("resource")
    acl = kwargs.get("acl", "private")
    bucket_name = kwargs.get("bucket_name")

    if kwargs.get("random_name"):
        bucket_name = gen_random_name(bucket_name)

    response = resource.Bucket(bucket_name).create(ACL=acl)
    return response


def neo_create_bucket(**kwargs):
    """Create a bucket with headers.

    :param auth: Tuple, consists of auth object and endpoint string
    :param acl: Input for canned ACL, defaults to "private"
    :param policy_id: String represent `x-gmt-policyid` or determines how data in the bucket will be distributed, defaults to None
    :param bucket_name: Bucket name
    :param random: A flag for deciding that a bucket name should be suffixed by random string or not, defaults to False
    """
    auth = kwargs.get("auth")
    acl = kwargs.get("acl", "private")
    policy_id = kwargs.get("policy_id", "")
    bucket_name = kwargs.get("bucket_name")

    if kwargs.get("random_name"):
        bucket_name = gen_random_name(bucket_name)

    endpoint = auth_lib.get_endpoint("storage", bucket_name)
    headers = {"x-gmt-policyid": policy_id, "x-amz-acl": acl}

    response = requests.put(endpoint, auth=auth, headers=headers)
    return response


def remove_bucket(resource, bucket_name):
    """Remove a bucket."""
    response = resource.Bucket(bucket_name).delete()
    return response


def get_objects(resource, bucket_name, prefix=""):
    """List objects inside a bucket"""
    client = resource.meta.client
    next_token = ""
    content = []
    directory = []
    add_list = lambda keys, values: keys.extend(values) if values is not None else None

    while True:
        response = client.list_objects_v2(
            Bucket=bucket_name,
            Prefix=prefix,
            Delimiter="/",
            ContinuationToken=next_token,
        )
        add_list(content, response.get("Contents"))
        add_list(directory, response.get("CommonPrefixes"))
        next_token = response.get("NextContinuationToken")

        if next_token is None:
            return {"Contents": content, "CommonPrefixes": directory}


def get_files(resource, bucket_name, prefix=""):
    """List only files inside a bucket (DIR excluded)."""
    files = []
    bucket = resource.Bucket(bucket_name)
    for file_ in bucket.objects.filter(Prefix=prefix):
        files.append(file_)
    return files


def is_exists(resource, bucket_name, object_name):
    response = get_objects(resource, bucket_name, prefix=object_name)
    if not response["Contents"]:
        return False

    if object_name in [x["Key"] for x in response["Contents"]]:
        return True
    else:
        return False


def remove_object(resource, bucket_name, object_name):
    """Remove an object in a bucket."""
    if is_exists(resource, bucket_name, object_name):
        response = resource.Object(bucket_name, object_name).delete()
        return response
    else:
        raise ValueError(f"Object not exists: {object_name}")


def download_object(resource, bucket_name, object_name):
    """Download an object in a bucket."""
    if not is_exists(resource, bucket_name, object_name):
        raise ValueError(f"Object not exists: {object_name}")

    if os.path.dirname(object_name):
        # if object contains '/'
        os.makedirs(os.path.dirname(object_name), exist_ok=True)
    resource.Object(bucket_name, object_name).download_file(f"{object_name}")


def list_download(resources, bucket_name, prefix, list_objects=None):
    """List of objects to download"""
    if list_objects is None:
        list_objects = []

    status = get_objects(resources, bucket_name, prefix)
    for obj in status["Contents"]:
        if obj["Key"][-1] != "/":
            list_objects.append(obj["Key"])
    for dir in status["CommonPrefixes"]:
        list_download(resources, bucket_name, dir["Prefix"], list_objects)
    return list_objects


def upload_object(**kwargs):
    """Upload an object into bucket."""
    # use local filename if not supplied
    filename = kwargs.get("local_path", "")
    local_path = kwargs.get("local_path")

    if kwargs.get("object_name"):
        filename = kwargs.get("object_name")

    resource = kwargs.get("resource")
    bucket_name = kwargs.get("bucket_name")
    resource_upload = resource.Object(bucket_name, filename)
    if kwargs.get("content_type"):
        resource_upload.upload_file(
            Filename=local_path, ExtraArgs={"ContentType": kwargs.get("content_type")}
        )
    else:
        resource_upload.upload_file(Filename=local_path)


def upload_bin_object(**kwargs):
    """Upload an binary object into bucket."""
    fileobj = kwargs.get("fileobj")
    filename = kwargs.get("object_name")

    resource = kwargs.get("resource")
    bucket_name = kwargs.get("bucket_name")
    resource_upload = resource.Object(bucket_name, filename)

    if kwargs.get("content_type"):
        resource_upload.upload_fileobj(
            Fileobj=fileobj, ExtraArgs={"ContentType": kwargs.get("content_type")}
        )
    else:
        resource_upload.upload_fileobj(Fileobj=fileobj)


def list_multipart_upload(resource, bucket_name, prefix=""):
    """List in-progress multipart uploads"""
    client = resource.meta.client
    mpupload = client.list_multipart_uploads(Bucket=bucket_name, Prefix=prefix)

    return {
        "Uploads": mpupload.get("Uploads"),
        "CommonPrefixes": mpupload.get("CommonPrefixes"),
    }


def list_part(resource, bucket_name, object_name, upload_id):
    """List in-progress part in multipart upload"""
    client = resource.meta.client
    mpupload = client.list_parts(
        Bucket=bucket_name, Key=object_name, UploadId=upload_id
    )
    return mpupload.get("Parts")


def abort_multipart_upload(resource, bucket_name, object_name, upload_id):
    """Abort in-progress multipart upload"""
    mpupload = resource.MultipartUpload(bucket_name, object_name, upload_id)
    return mpupload.abort()


def complete_multipart_upload(resource, bucket_name, object_name, upload_id):
    """Complete a multipart upload by assembling uploaded parts"""
    mpupload = resource.MultipartUpload(bucket_name, object_name, upload_id)
    parts = list_part(resource, bucket_name, object_name, upload_id)

    multipart = []
    for part in parts:
        multipart.append(
            {"ETag": part.get("ETag"), "PartNumber": part.get("PartNumber")}
        )
    return mpupload.complete(MultipartUpload={"Parts": multipart})


def copy_object(resource, src_bucket, src_object_name, dest_bucket, dest_object_name):
    """Copy an object into other bucket."""

    if not dest_object_name:
        # use source object name if not supplied
        dest_object_name = src_object_name

    copy_source = {"Bucket": src_bucket, "Key": src_object_name}
    resource.Object(dest_bucket, dest_object_name).copy(copy_source)


def move_object(resource, src_bucket, src_object_name, dest_bucket, dest_object_name):
    """Move an object into other bucket.
    Using copy then remove operation.
    """
    copy_object(resource, src_bucket, src_object_name, dest_bucket, dest_object_name)
    remove_object(resource, src_bucket, src_object_name)


def bucket_usage(resource, bucket_name):
    """Calculate bucket usage."""
    objects = get_files(resource, bucket_name)
    total_objects = len(objects)
    total_size = 0
    for obj in objects:
        total_size += obj.size
    return total_size, total_objects


def disk_usage(resource):
    """Calculate disk usage."""
    all_buckets = buckets(resource)
    bucket_names = [bucket.name for bucket in all_buckets]

    disk_usages = []
    for bucket_name in bucket_names:
        usage = bucket_usage(resource, bucket_name)
        disk_usages.append([bucket_name, usage])

    return disk_usages


def get_cors(bucket):
    try:
        cors = bucket.Cors().cors_rules
    except Exception:
        cors = None
    return cors


def get_policy(bucket):
    try:
        policy = bucket.Policy().policy
    except Exception:
        policy = None
    return policy


def get_location(client, bucket_name):
    response = client.get_bucket_location(Bucket=bucket_name)
    location = response["LocationConstraint"]
    return location


def get_expiration(client, bucket_name):
    try:
        exp = client.get_bucket_lifecycle(Bucket=bucket_name)
    except Exception:
        exp = None
    return exp


def get_grant_name(grant):
    """Get grant name based on Grantee type."""
    grant_name = ""
    if grant["Grantee"]["Type"] == "Group":
        uri = grant["Grantee"]["URI"]
        grant_name = uri.rsplit("/", 1)[-1]

    if grant["Grantee"]["Type"] == "CanonicalUser":
        grant_name = grant["Grantee"]["DisplayName"]

    return grant_name


def get_grants(obj):
    """Get grants info of bucket or object."""
    grants = obj.Acl().grants
    grantees = []

    for grant in grants:
        name = get_grant_name(grant)
        permission = grant["Permission"]
        grantee = [name, permission]
        grantees.append(grantee)

    return grantees


def bucket_gmt_policy(bucket_name, auth):
    """Get bucket GMT-Policy"""
    policy_id = gmt.policy_id(bucket_name, auth)
    description = gmt.policy_description(policy_id)
    return description


def bucket_info(resource, bucket_name, auth):
    """Info of bucket."""
    bucket = resource.Bucket(bucket_name)
    client = resource.meta.client

    gmt_policy = bucket_gmt_policy(bucket_name, auth)
    info = {
        "ACL": get_grants(bucket),
        "CORS": get_cors(bucket),
        "Policy": get_policy(bucket),
        "Expiration": get_expiration(client, bucket_name),
        "Location": get_location(client, bucket_name),
    }
    if gmt_policy:
        info["GmtPolicy"] = gmt_policy

    return info


def object_info(resource, bucket_name, object_name):
    """Info of object."""
    obj = resource.Object(bucket_name=bucket_name, key=object_name)
    storage_class = obj.storage_class
    content_type = obj.content_type
    grantees = get_grants(obj)

    info = {
        "ACL": grantees,
        "Size": obj.content_length,
        "LastModified": obj.last_modified,
        "MD5": obj.e_tag,
        "MimeType": content_type,
        "StorageClass": storage_class,
    }
    return info


def set_acl(**kwargs):
    """Set ACL of object or object."""
    resource = kwargs.get("resource")
    bucket_name = kwargs.get("bucket_name")
    object_name = kwargs.get("object_name")

    if kwargs.get("acl_type") == "object":
        obj = resource.Object(bucket_name=bucket_name, key=object_name)
    if kwargs.get("acl_type") == "bucket":
        obj = resource.Bucket(bucket_name)

    response = obj.Acl().put(
        ACL=kwargs.get("acl"),
        GrantFullControl=utils.set_id(kwargs.get("grant_fullcontrol")),
        GrantRead=utils.set_id(kwargs.get("grant_read")),
        GrantReadACP=utils.set_id(kwargs.get("grant_readACP")),
        GrantWrite=utils.set_id(kwargs.get("grant_write")),
        GrantWriteACP=utils.set_id(kwargs.get("grant_writeACP")),
    )
    return response


def generate_url(resource, bucket_name, object_name, expire=3600):
    """Generate URL for bucket or object."""
    client = resource.meta.client
    url = client.generate_presigned_url(
        "get_object",
        Params={"Bucket": bucket_name, "Key": object_name},
        ExpiresIn=expire,
    )
    return url


def mkdir(resource, bucket_name, dir_name):
    """Create directory inside bucket"""
    client = resource.meta.client
    if not dir_name.endswith("/"):
        dir_name = f"{dir_name}/"

    response = client.put_object(Bucket=bucket_name, Body="", Key=dir_name)
    return response



import mock
import pytest
import tzlocal
import xmltodict
from obs.libs import utils
from datetime import timezone


def test_size():
    # 100*13 is used to get size with unit YiB
    assert utils.sizeof_fmt(100 ** 13) == "82.7 YiB"


def test_date(monkeypatch):
    monkeypatch.setattr(tzlocal, "get_localzone", lambda: timezone.utc)
    assert utils.human_date(0) == f"1970-01-01 00:00:00+0000 (UTC)"


def test_check():
    response = {"reason": "nonsense", "status_code": "valid", "url": ".net"}
    with pytest.raises(ValueError, match=(f"nonsense")):
        utils.check(response)


def fake_response():
    response = mock.Mock()
    response.text = "valid"
    return response


def fake_parse(text):
    response = {"Error": {"Code": "foo", "Message": "False"}}
    return response


def test_check_plain(monkeypatch):
    monkeypatch.setattr(xmltodict, "parse", fake_parse)
    with pytest.raises(ValueError, match=(f"foo: False")):
        utils.check_plain(fake_response())



import os
import yaml
import requests

from obs.libs import auth as auth_lib


def policies_file():
    policy_file = os.environ.get("OBS_USER_GMT_POLICY")
    return policy_file


def is_policy_exists():
    policy_file = policies_file()
    return os.path.isfile(policy_file)


def get_policies():
    """Get policies."""
    policy_file = policies_file()
    if policy_file != "notset" and is_policy_exists():
        policies = yaml.safe_load(open(policy_file))
        return policies
    else:
        return "notset"


def policy_id(bucket_name, auth):
    """Get GMT-Policy id from S3 API response headers."""

    endpoint = auth_lib.get_endpoint("storage", bucket_name)
    response = requests.get(endpoint, auth=auth)
    policy_id = response.headers.get("x-gmt-policyid")

    return policy_id


def policy_description(policy_id):
    """Get GMT-Policy description."""
    policies = get_policies()
    # id not found also will return None
    # so gmt policy will not be shown
    description = None

    if policies == "notset":
        return

    for zone in policies:
        policyid, _description, _ = policies[zone].values()
        if policyid == policy_id:
            description = _description
            break

    if description == "":
        description = "No description"

    return description



import pytest
import pathlib

from dotenv import load_dotenv
from obs.api.app import create_app


@pytest.fixture
def client():
    app = create_app()
    client = app.test_client()

    current_path = pathlib.Path(__file__)
    dotenv_path = "tests/api/test_api.env"
    load_dotenv(dotenv_path=dotenv_path)

    yield client



import os


class MetaFlaskEnv(type):
    def __init__(cls, name, bases, dict):
        """
        MetaFlaskEnv class initializer.

        This function will get called when a new class which utilizes this metaclass is defined,
        as opposed to when it is initialized.
        """
        super(MetaFlaskEnv, cls).__init__(name, bases, dict)

        # Get our internal settings
        prefix = dict.get("ENV_PREFIX", "")

        # Override default configuration from environment variables
        for key, value in os.environ.items():
            # Only include environment keys that start with our prefix (if we have one)
            if not key.startswith(prefix):
                continue

            # Strip the prefix from the environment variable name
            key = key[len(prefix) :]

            # If value is "true" or "false", parse as a boolean
            # Otherwise, if it contains a "." then try to parse as a float
            # Otherwise, try to parse as an integer
            # If all else fails, just keep it a string
            if "PASSWORD" in key:
                value = str(value)
            elif value.lower() in ("true", "false"):
                value = True if value.lower() == "true" else False
            elif "." in value:
                try:
                    value = float(value)
                except ValueError:
                    pass
            else:
                try:
                    value = int(value)
                except ValueError:
                    pass

            # Update our config with the value from `os.environ`
            setattr(cls, key, value)



import mock
from obs.libs import user


def test_dump():
    usr = user.UserProfile
    options = [
        ("userId", "User ID"),
        ("groupId", "Group ID"),
        ("userType", "User Type"),
        ("fullName", "Full Name"),
        ("emailAddr", "Email"),
        ("address1", "Address"),
        ("city", "City"),
        ("state", "State"),
        ("zip", "Zip"),
        ("country", "Country"),
        ("phone", "Phone"),
        ("website", "Website"),
        ("active", "Active"),
        ("ldapEnabled", "LDAP Enabled"),
    ]

    assert user.UserProfile().dump(options, usr) == {
        "userId": "",
        "groupId": "",
        "userType": "User",
        "fullName": "",
        "emailAddr": "",
        "address1": "",
        "city": "",
        "state": "",
        "zip": "",
        "country": "",
        "phone": "",
        "website": "",
        "active": True,
        "ldapEnabled": False,
    }


def fake_list():
    user1 = {
        "userId": "jerrygarcia",
        "fullName": "Jerry Garcia",
        "emailAddr": "garcia@bgn.net",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "active": True,
    }
    user2 = {
        "userId": "johnthompson",
        "fullName": "John Thompson",
        "emailAddr": "",
        "address1": "",
        "city": "",
        "active": True,
    }

    ls = mock.Mock()
    ls.user.list.return_value = [user1, user2]
    return ls


def test_list():
    assert user.list_user(fake_list(), "group") == [
        {
            "userId": "jerrygarcia",
            "fullName": "Jerry Garcia",
            "emailAddr": "garcia@bgn.net",
            "address1": "456 Shakedown St.",
            "city": "Portsmouth",
            "active": True,
        },
        {
            "userId": "johnthompson",
            "fullName": "John Thompson",
            "emailAddr": "",
            "address1": "",
            "city": "",
            "active": True,
        },
    ]


def fake_info():
    user = {
        "userId": "johnthompson",
        "fullName": "John Thompson",
        "emailAddr": "jgarc@geemail.com",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "groupId": "testing",
        "canonicalUserId": "2c82bdc930155e8dc6860bfake",
        "active": True,
    }
    info = mock.Mock()
    info.user.return_value = user
    return info


def test_info():
    assert user.info(fake_info(), "ocean", "nature") == {
        "userId": "johnthompson",
        "fullName": "John Thompson",
        "emailAddr": "jgarc@geemail.com",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "groupId": "testing",
        "canonicalUserId": "2c82bdc930155e8dc6860bfake",
        "active": True,
    }


def fake_client():
    client = mock.Mock()
    client.user.return_value = "Done"
    return client


def test_create():
    assert user.create(fake_client(), "data") == "Done"


def test_remove():
    assert user.remove(fake_client(), "user", "group")



import click


from obs.cli.storage import config
from obs.cli.storage import commands
from obs.cli.admin import commands as commands_admin


@click.group(invoke_without_command=True)
@click.option(
    "--configure",
    is_flag=True,
    default=False,
    help="Configure object storage service values",
)
@click.version_option()
def cli(configure):
    """neo-obs

    Command line tool for neo object storage.
    """
    if configure:
        config.run_configure()


cli.add_command(commands.storage)
cli.add_command(commands_admin.admin)

if __name__ == "__main__":
    cli()



from obs.libs import qos
from obs.libs import user
from obs.libs import credential
from obs.libs import auth as client
from obs.libs import admin as admin_usage
from obs.libs import utils
from obs.api.app.helpers.rest import response
from flask import current_app
from flask_restful import Resource, reqparse, inputs


def get_client():
    return client.admin_client()


class user_api(Resource):
    def get(self):
        parser = reqparse.RequestParser()
        parser.add_argument("user_type", type=str, default="all")
        parser.add_argument("user_status", type=str, default="active")
        parser.add_argument("limit", type=str, default="")
        parser.add_argument("userId", type=str)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            if args["userId"]:
                users = user.info(get_client(), args["userId"], args["groupId"])
                if "reason" in users:
                    if users["status_code"] == 204:
                        current_app.logger.error(f"User {args['userId']} not found")
                        return response(200, f"User {args['userId']} not found")

                    current_app.logger.error(users["reason"])
                    return response(users["status_code"], message=users["reason"])
                return response(200, data=users)

            user_list = user.list_user(
                get_client(),
                args["groupId"],
                args["user_type"],
                args["user_status"],
                args["limit"],
            )

            if "reason" in user_list:
                current_app.logger.error(user_list["reason"])
                return response(user_list["status_code"], message=user_list["reason"])

            return response(200, data=user_list)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def post(self):
        options = {
            "userId": "",
            "groupId": "",
            "userType": "User",
            "fullName": "",
            "emailAddr": "",
            "address1": "",
            "city": "",
            "state": "",
            "zip": "",
            "country": "",
            "phone": "",
            "website": "",
            "active": True,
            "ldapEnabled": False,
        }

        parser = reqparse.RequestParser()
        for index, option in options.items():
            if index in ["userId", "groupId", "fullName"]:
                parser.add_argument(index, type=str, required=True)
            elif index in ["active", "ldapEnabled"]:
                parser.add_argument(index, type=inputs.boolean)
            else:
                parser.add_argument(index, type=str)
        parser.add_argument("quotaLimit", type=int)
        args = parser.parse_args()

        try:
            for index, option in options.items():
                if args[index] not in (option, None):
                    options[index] = args[index]
            options = utils.user_sanitize(options)
            status = user.create(get_client(), options)
            if args["quotaLimit"]:
                status = qos.set(
                    get_client(), args["userId"], args["groupId"], args["quotaLimit"]
                )
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])
            else:
                return response(201, f"User {options['userId']} created successfully.")
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def put(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        parser.add_argument("suspend", type=inputs.boolean, required=True)
        args = parser.parse_args()

        try:
            msg = "suspended" if args["suspend"] is True else "unsuspended"
            users = user.info(get_client(), args["userId"], args["groupId"])
            if "reason" in users:
                if users["status_code"] == 204:
                    current_app.logger.error(f"User {args['userId']} not found")
                    return response(200, f"User {args['userId']} not found")

            if users["active"] == f"{not args['suspend']}".lower():
                return response(400, f"User already {msg}")

            # canocicalUserId can't be included when updating user
            del users["canonicalUserId"]
            users["active"] = not args["suspend"]
            status = user.update(get_client(), users)
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            message = f"User has been {msg}"
            return response(200, message)
        except Exception as e:
            current_app.logger.error(f"{e}")
            response(500, f"{e}")

    def delete(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            status = user.remove(get_client(), args["userId"], args["groupId"])
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(200, f"User {args['userId']} deleted successfully.")
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class qos_api(Resource):
    def get(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            infos = qos.info(get_client(), args["userId"], args["groupId"])
            infos["Storage Limit"] = (
                infos["qosLimitList"][0]["value"] * 1024
                if infos["qosLimitList"][0]["value"] != -1
                else "unlimited"
            )

            if "reason" in infos:
                current_app.logger.error(infos["reason"])
                return response(infos["status_code"], message=infos["reason"])

            return response(200, data=infos)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def post(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        parser.add_argument("limit", type=int, required=True)
        args = parser.parse_args()

        try:
            status = qos.set(
                get_client(), args["userId"], args["groupId"], args["limit"]
            )
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(
                201, f"User {args['userId']} quota changed successfully.", status
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def delete(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            status = qos.rm(get_client(), args["userId"], args["groupId"])
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(
                200, f"User {args['userId']} quota changed to unlimited.", status
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class cred_api(Resource):
    def get(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            cred_list = credential.list(get_client(), args["userId"], args["groupId"])
            if "reason" in cred_list:
                current_app.logger.error(cred_list["reason"])
                return response(cred_list["status_code"], message=cred_list["reason"])

            return response(200, data=cred_list)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def post(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            status = credential.create(get_client(), args["userId"], args["groupId"])
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(
                201, f"User {args['userId']} new credential created successfully."
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def put(self):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("status", type=str, required=True, default="true")
        args = parser.parse_args()

        try:
            stats = "activated" if args["status"].lower() == "true" else "deactivated"
            status = credential.status(get_client(), args["access_key"], args["status"])
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(200, f"Credential status has been {stats}.")
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def delete(self):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        args = parser.parse_args()

        try:
            status = credential.rm(get_client(), args["access_key"])
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(
                200, f"Access key {args['access_key']} deleted successfully."
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class user_usage(Resource):
    def get(self):
        parser = reqparse.RequestParser()
        parser.add_argument("userId", type=str, required=True)
        parser.add_argument("groupId", type=str, required=True)
        args = parser.parse_args()

        try:
            status = admin_usage.usage(get_client(), args["userId"], args["groupId"])
            if "reason" in status:
                current_app.logger.error(status["reason"])
                return response(status["status_code"], message=status["reason"])

            return response(200, data=status[0])
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")



import os
import re
import boto3
import zipfile
import tempfile
import xmltodict

from obs.libs import bucket
from obs.libs import gmt
from obs.libs import auth
from obs.libs import utils
from distutils.util import strtobool
from requests_aws4auth import AWS4Auth
from obs.api.app.helpers.rest import response
from werkzeug.utils import secure_filename
from flask import request, current_app, send_from_directory
from flask_restful import Resource, reqparse, inputs


def get_resources(access_key, secret_key):
    endpoint = auth.get_endpoint("storage")
    sess = boto3.Session(aws_access_key_id=access_key, aws_secret_access_key=secret_key)
    s3_resource = sess.resource("s3", endpoint_url=endpoint)
    return s3_resource


def get_plain_auth(access_key, secret_key):
    auth = AWS4Auth(access_key, secret_key, "eu-west-1", "s3")
    return auth


def list_objects(buckets, contents="Contents", date="LastModified"):
    objects = []
    if buckets["CommonPrefixes"]:
        for prefix in buckets["CommonPrefixes"]:
            objects.append({"directory": f"{prefix['Prefix']}"})

    if buckets[contents]:
        for content in buckets[contents]:
            content[date] = f"{content[date]:%Y-%m-%d %H:%M:%S}"
            objects.append(content)
    return objects


class list(Resource):
    def get(self, prefix=""):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("bucket_name", type=str)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        bucket_name = args["bucket_name"]
        if args["bucket_name"]:
            bucket_name, prefix = utils.get_bucket_key(args["bucket_name"])

        try:
            if args["bucket_name"]:
                buckets = bucket.get_objects(
                    get_resources(args["access_key"], secret_key), bucket_name, prefix
                )
                objects = list_objects(buckets)
                if not objects:
                    return response(200, f"Bucket is Empty.")
                return response(200, data=objects)

            buckets = bucket.buckets(get_resources(args["access_key"], secret_key))
            all_bucket = []
            for index, buck in enumerate(buckets):
                all_bucket.append(
                    {
                        "name": buck.name,
                        "creation_date": f"{buck.creation_date:%Y-%m-%d %H:%M:%S}",
                    }
                )
            if not all_bucket:
                return response(200, f"Storage is Empty.")
            return response(200, data=all_bucket)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class bucket_api(Resource):
    def get(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            bucket_info = bucket.bucket_info(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                get_plain_auth(args["access_key"], secret_key),
            )
            return response(200, data=bucket_info)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def post(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("acl", type=str, default="private")
        parser.add_argument("policy_id", type=str, default="")
        parser.add_argument("random_name", type=inputs.boolean, default=False)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            bucket_name = utils.sanitize("bucket", bucket_name)
            if not 2 < len(bucket_name) < 64:
                return response(
                    400, f"'{bucket_name}' too short or too long for bucket name"
                )

            attr = {
                "auth": get_plain_auth(args["access_key"], secret_key),
                "resource": get_resources(args["access_key"], secret_key),
                "bucket_name": bucket_name,
                "acl": args["acl"],
                "policy_id": args["policy_id"],
                "random_name": args["random_name"],
            }
            if utils.compatibility():
                responses = bucket.neo_create_bucket(**attr)
                if responses.text:
                    error = xmltodict.parse(responses.text)
                    return response(400, error["Error"]["Message"])
                return response(201, f"Bucket created successfully.", responses.text)

            else:
                responses = bucket.create_bucket(**attr)
                return response(201, f"Bucket created successfully.", responses)

        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def delete(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            result = bucket.remove_bucket(
                get_resources(args["access_key"], secret_key), bucket_name
            )
            return response(200, f"Bucket {bucket_name} deleted successfully.", result)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class object_api(Resource):
    def get(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            object_info = bucket.object_info(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
            )
            for key, value in object_info.items():
                object_info[key] = f"{value}"
            return response(200, data=object_info)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def delete(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            result = bucket.remove_object(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
            )
            return response(
                200, f"Object {args['object_name']} deleted successfully.", result
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class move_object(Resource):
    def post(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, required=True)
        parser.add_argument("move_to", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            bucket.move_object(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
                args["move_to"],
                None,
            )
            return response(201, f"Object {args['object_name']} moved successfully.")
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class copy_object(Resource):
    def post(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, required=True)
        parser.add_argument("copy_to", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            bucket.copy_object(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
                args["copy_to"],
                None,
            )
            return response(201, f"Object {args['object_name']} copied successfully.")
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


def archive(resources, dir_name, bucket_name, prefix):
    objects = bucket.list_download(resources, bucket_name, prefix)
    with zipfile.ZipFile(f"{dir_name[:-1]}.zip", "w") as zip_file:
        for obj in objects:
            zip_file.write(obj, bucket.download_object(resources, bucket_name, obj))
    return f"{dir_name[:-1]}.zip"


class download_object(Resource):
    def get(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, default="")
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        with tempfile.TemporaryDirectory() as tempdir:
            try:
                os.chdir(tempdir)
                resources = get_resources(args["access_key"], secret_key)

                if args["object_name"] == "":
                    name = archive(resources, f"{bucket_name}/", bucket_name, "")
                elif args["object_name"][-1] == "/":
                    name = archive(
                        resources, args["object_name"], bucket_name, args["object_name"]
                    )
                else:
                    bucket.download_object(resources, bucket_name, args["object_name"])
                    name = args["object_name"]
                file = send_from_directory(tempdir, name, as_attachment=True)
                return file
            except Exception as e:
                current_app.logger.error(f"{e}")
                return response(500, f"{e}")


class upload_object(Resource):
    def get(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, default="")
        parser.add_argument("upload_id", type=str)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            if args["upload_id"]:
                parts = bucket.list_part(
                    get_resources(args["access_key"], secret_key),
                    bucket_name,
                    args["object_name"],
                    args["upload_id"],
                )
                if parts is None:
                    return response(404, f"Part not found, please abort this object.")
                for part in parts:
                    part["LastModified"] = f'{part["LastModified"]:%Y-%m-%d %H:%M:%S}'
                return response(201, data=parts)

            mpu = bucket.list_multipart_upload(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
            )
            list_objects(mpu, "Uploads", "Initiated")
            return response(200, data=mpu)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def delete(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, required=True)
        parser.add_argument("upload_id", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            mpu = bucket.abort_multipart_upload(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
                args["upload_id"],
            )
            return response(
                200, f"Multipart upload of {args['object_name']} has been aborted.", mpu
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def put(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str, required=True)
        parser.add_argument("upload_id", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            mpu = bucket.complete_multipart_upload(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["object_name"],
                args["upload_id"],
            )
            return response(
                200, f"Multipart upload for {args['object_name']} has been completed."
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")

    def post(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("object_name", type=str)
        parser.add_argument("acl", type=str)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        file = request.files["files"]
        filename = secure_filename(file.filename)
        object_name = args["object_name"] if args["object_name"] else filename

        try:
            object_name = utils.sanitize("upload", object_name)
            result = bucket.upload_bin_object(
                resource=get_resources(args["access_key"], secret_key),
                bucket_name=bucket_name,
                fileobj=file,
                object_name=object_name,
                content_type=file.content_type,
            )
            if args["acl"]:
                bucket.set_acl(
                    resource=get_resources(args["access_key"], secret_key),
                    bucket_name=bucket_name,
                    object_name=object_name,
                    acl_type="object",
                    acl=args["acl"],
                )
            return response(201, f"Object {object_name} uploaded successfully.", result)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class usage(Resource):
    def get(self):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("bucket_name", type=str)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            if args["bucket_name"]:
                total_size, total_objects = bucket.bucket_usage(
                    get_resources(args["access_key"], secret_key), args["bucket_name"]
                )
                bucket_usage = {
                    "name": args["bucket_name"],
                    "size": total_size,
                    "objects": total_objects,
                }
                return response(200, data=bucket_usage)

            disk_usage = {"bucket": [], "total_usage": 0}
            disk_usages = bucket.disk_usage(
                get_resources(args["access_key"], secret_key)
            )
            for usage in disk_usages:
                bucket_name = usage[0]
                total_size, total_objects = usage[1]
                disk_usage["total_usage"] += total_size
                disk_usage["bucket"].append(
                    {"name": bucket_name, "size": total_size, "objects": total_objects}
                )
            disk_usage["total_usage"] = f"{disk_usage['total_usage']}"
            return response(200, data=disk_usage)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class acl(Resource):
    def post(self):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("bucket_name", type=str, required=True)
        parser.add_argument("object_name", type=str)
        parser.add_argument("acl", type=str, default="")
        parser.add_argument("grant_fullcontrol", type=str)
        parser.add_argument("grant_read", type=str)
        parser.add_argument("grant_readACP", type=str)
        parser.add_argument("grant_write", type=str)
        parser.add_argument("grant_writeACP", type=str)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            name = args["object_name"] if args["object_name"] else args["bucket_name"]
            attr = {arg: val for arg, val in args.items() if "key" not in arg}
            attr["resource"] = get_resources(args["access_key"], secret_key)
            attr["acl_type"] = "object" if args["object_name"] else "bucket"

            result = bucket.set_acl(**attr)
            return response(200, f"Added access to {attr['acl_type']} {name}.", result)
        except Exception as e:
            current_app.logger.error(f"{e}", exc_info=1)
            return response(500, f"{e}")


class presign(Resource):
    def get(self, bucket_name, object_name):
        parser = reqparse.RequestParser()
        parser.add_argument("expire", type=int)
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            url = bucket.generate_url(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                object_name,
                args["expire"],
            )
            return response(200, data=url)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class mkdir(Resource):
    def post(self, bucket_name):
        parser = reqparse.RequestParser()
        parser.add_argument("access_key", type=str, required=True)
        parser.add_argument("secret_key", type=str, required=True)
        parser.add_argument("directory", type=str, required=True)
        args = parser.parse_args()
        secret_key = args["secret_key"].replace(" ", "+")

        try:
            result = bucket.mkdir(
                get_resources(args["access_key"], secret_key),
                bucket_name,
                args["directory"],
            )
            return response(
                201, f"Directory {args['directory']} added successfully.", result
            )
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")


class gmt_policy(Resource):
    def get(self):
        try:
            msg = []
            policies = gmt.get_policies()

            for zone in policies:
                policy_id, description, _ = policies[zone].values()
                if not description:
                    description = "No description"
                msg.append({"Name": zone, "Id": policy_id, "Description": description})
            return response(200, data=msg)
        except Exception as e:
            current_app.logger.error(f"{e}")
            return response(500, f"{e}")



import os
from obs.libs import config


def test_config_file(monkeypatch):
    monkeypatch.setattr(os.path, "expanduser", lambda comment: "home/hors")
    assert config.config_file() == "home/hors/.config/neo-obs/obs.env"


def test_exists(monkeypatch):
    monkeypatch.setattr(config, "config_file", lambda: None)
    monkeypatch.setattr(os.path, "isfile", lambda file: True)
    assert config.is_config_exists() is True



import requests
import json
import os
import logging
import coloredlogs


def log_rep(stdin):
    coloredlogs.install()
    logging.info(stdin)


def log_err(stdin):
    coloredlogs.install()
    logging.error(stdin)


def send_http(url, data, headers=None):
    json_data = json.dumps(data)
    try:
        send = requests.post(url, data=json_data, headers=headers)
        respons = send.json()
        response_time = send.elapsed.total_seconds()
    except requests.exceptions.RequestException:
        respons = {
            "result": False,
            "Error": "Failed to establish a new connection",
            "description": None,
        }
        return respons
    else:
        result = {
            "result": respons["result"],
            "time": response_time,
            "data": respons["data"],
            "status": respons["status"],
        }
        return result


def get_http(url, params=None):
    try:
        response = requests.get(url, params)
    except requests.exceptions.RequestException as e:
        log_err(e)
        return False
    except Exception as a:
        log_err(a)
        return False
    else:
        data = response.json()
        return data



import click
import sys

from obs.libs import auth
from obs.cli.storage import bucket
from obs.cli.storage import gmt
from obs.libs import utils
from obs.libs import config


def warn_inexsit_config():
    msg = (
        f"Configuration file not available.\n"
        f"Consider running 'obs --configure' to create one"
    )
    click.secho(msg, fg="yellow", bold=True, err=True)


def get_resources():
    try:
        config.load_config_file()
        s3_resource = auth.resource()
        return s3_resource
    except Exception as exc:
        click.secho(str(exc), fg="yellow", bold=True, err=True)
        warn_inexsit_config()
        sys.exit(1)


def get_plain_auth():
    try:
        config.load_config_file()
        plain_auth = auth.plain_auth()
        return plain_auth
    except Exception as exc:
        click.secho(str(exc), fg="yellow", bold=True, err=True)
        warn_inexsit_config()
        sys.exit(1)


@click.group()
def storage():
    """Manage user storage."""


@storage.command("ls")
@click.argument("uri", default="", required=False)
def list(uri):
    """List bucket or object."""
    s3_resource = get_resources()
    if uri:
        bucket.get_objects(s3_resource, uri=uri)
    else:
        bucket.buckets(s3_resource)


@storage.command("rm")
@click.argument("uri")
def remove(uri):
    """Remove bucket or object."""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)

    if not prefix:
        bucket.remove_bucket(s3_resource, bucket_name=bucket_name)
    if bucket_name and prefix:
        bucket.remove_object(s3_resource, bucket_name=bucket_name, object_name=prefix)


@storage.command("mb")
@click.argument("bucket_name", default="")
@click.option(
    "--acl", "acl", default="private", required=False, help="Access Control List"
)
@click.option(
    "--policy-id",
    "policy_id",
    default="",
    required=False,
    help="Data distribution in bucket",
)
@click.option(
    "-r",
    "--random",
    "random_name",
    default=False,
    is_flag=True,
    help="Generate random name",
)
def make_bucket(bucket_name, acl, policy_id, random_name):
    """Create bucket."""
    plain_auth = get_plain_auth()
    s3_resource = get_resources()
    bucket.create_bucket(
        auth=plain_auth,
        resource=s3_resource,
        bucket_name=bucket_name,
        acl=acl,
        policy_id=policy_id,
        random_name=random_name,
    )


@storage.command("get")
@click.argument("uri")
def get_object(uri):
    """Download object in bucket."""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)
    bucket.download_object(s3_resource, bucket_name=bucket_name, object_name=prefix)


@storage.command("put")
@click.argument("local_path", default="")
@click.argument("uri")
def put_object(local_path, uri):
    """Upload object to bucket."""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)
    bucket.upload_object(
        resource=s3_resource,
        bucket_name=bucket_name,
        local_path=local_path,
        object_name=prefix,
    )


@storage.command("cp")
@click.argument("src_uri", default="")
@click.argument("dest_uri", default="")
def copy_object(src_uri, dest_uri):
    """Copy object to other bucket."""
    s3_resource = get_resources()

    src_bucket, src_prefix = utils.get_bucket_key(src_uri)
    dest_bucket, dest_prefix = utils.get_bucket_key(dest_uri)
    bucket.copy_object(s3_resource, src_bucket, src_prefix, dest_bucket, dest_prefix)


@storage.command("mv")
@click.argument("src_uri", default="")
@click.argument("dest_uri", default="")
def move_object(src_uri, dest_uri):
    """Move object into other bucket."""
    s3_resource = get_resources()

    src_bucket, src_prefix = utils.get_bucket_key(src_uri)
    dest_bucket, dest_prefix = utils.get_bucket_key(dest_uri)

    bucket.move_object(s3_resource, src_bucket, src_prefix, dest_bucket, dest_prefix)


@storage.command("du")
@click.argument("uri", default="", required=False)
def du(uri):
    """Show disk or bucket usage."""
    s3_resource = get_resources()

    bucket_name, prefix = utils.get_bucket_key(uri)
    if bucket_name:
        bucket.bucket_usage(s3_resource, bucket_name=bucket_name)
    else:
        bucket.disk_usage(s3_resource)


@storage.command("info")
@click.argument("uri")
def info(uri):
    """Display bucket or object info."""
    s3_resource = get_resources()
    plain_auth = get_plain_auth()

    bucket_name, prefix = utils.get_bucket_key(uri)
    if not prefix:
        bucket.bucket_info(s3_resource, bucket_name=bucket_name, auth=plain_auth)
    if bucket_name and prefix:
        bucket.object_info(s3_resource, bucket_name=bucket_name, object_name=prefix)


@storage.command("acl")
@click.argument("uri")
@click.argument("acl", default="")
@click.option(
    "--read", "read", default=None, metavar="", help="granted read access to user id"
)
@click.option(
    "--write", "write", default=None, metavar="", help="granted write access to user id"
)
@click.option(
    "--read-acp",
    "readACP",
    default=None,
    metavar="",
    help="granted readACP access to user id",
)
@click.option(
    "--write-acp",
    "writeACP",
    default=None,
    metavar="",
    help="granted writeACP access to user id",
)
@click.option(
    "--full-control",
    "fullcontrol",
    default=None,
    metavar="",
    help="granted full access to user id",
)
def set_acl(uri, acl, fullcontrol, read, readACP, write, writeACP):
    """Set ACL for bucket or object."""
    s3_resource = get_resources()

    bucket_name, prefix = utils.get_bucket_key(uri)

    opt = {
        "resource": s3_resource,
        "bucket_name": bucket_name,
        "acl": acl,
        "grant_fullcontrol": fullcontrol,
        "grant_read": read,
        "grant_readACP": readACP,
        "grant_write": write,
        "grant_writeACP": writeACP,
    }

    if not prefix:
        bucket.set_acl(**opt, acl_type="bucket")
    if bucket_name and prefix:
        bucket.set_acl(**opt, object_name=prefix, acl_type="object")


@storage.command("presign")
@click.argument("uri")
@click.option("--expire", "expire", type=int, help="Set expiration time [default:3600]")
def url(uri, expire):
    """Generate presign URL for object."""
    s3_resource = get_resources()

    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)
    bucket.generate_url(
        resource=s3_resource, bucket_name=bucket_name, object_name=prefix, expire=expire
    )


@storage.command("mkdir")
@click.argument("uri")
def mkdir(uri):
    """Create directory inside bucket"""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)

    bucket.mkdir(resource=s3_resource, bucket_name=bucket_name, dir_name=prefix)


@storage.command("gmt")
@click.option("--policy-id", "policy_id", is_flag=True, help="List Gmt Policy ID")
def gmt_cmd(policy_id):
    """Manage Cloudian extensions to S3."""
    if policy_id:
        config.load_config_file()
        gmt.show_policies()


@storage.group()
def mpu():
    """Manage multipart upload function"""


@mpu.command("ls")
@click.argument("uri")
def list_mpu(uri):
    """List in-progress multipart uploads"""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)

    bucket.list_multipart_upload(
        resource=s3_resource, bucket_name=bucket_name, prefix=prefix
    )


@mpu.command("part")
@click.argument("uri")
@click.argument("upload_id")
def list_part(uri, upload_id):
    """List in-progress part in multipart upload"""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)

    bucket.list_part(
        resource=s3_resource,
        bucket_name=bucket_name,
        object_name=prefix,
        upload_id=upload_id,
    )


@mpu.command("abort")
@click.argument("uri")
@click.argument("upload_id")
def abort_mpu(uri, upload_id):
    """Abort in-progress multipart upload"""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)

    bucket.abort_multipart_upload(
        resource=s3_resource,
        bucket_name=bucket_name,
        object_name=prefix,
        upload_id=upload_id,
    )


@mpu.command("complete")
@click.argument("uri")
@click.argument("upload_id")
def abort_mpu(uri, upload_id):
    """Complete a multipart upload by assembling uploaded parts"""
    s3_resource = get_resources()
    bucket_name, prefix = utils.get_bucket_key(uri)

    bucket.complete_multipart_upload(
        resource=s3_resource,
        bucket_name=bucket_name,
        object_name=prefix,
        upload_id=upload_id,
    )



import pytest
import io
import os
import mock
import bitmath
from datetime import datetime

import obs.libs.auth
import obs.libs.user
import obs.libs.utils
import obs.libs.config
import obs.cli.admin.commands as admin_client
from click.testing import CliRunner
from obs.cli.main import cli


def fake_client():
    pass


def fake_response():
    return {"reason": "error", "status_code": 101, "url": "http://testing"}


def load_config_file():
    pass


@pytest.fixture
def client(monkeypatch):
    monkeypatch.setattr(obs.libs.config, "load_config_file", load_config_file)
    monkeypatch.setattr(obs.libs.auth, "admin_client", fake_client)


def test_get_client(monkeypatch):
    monkeypatch.setattr(obs.libs.config, "config_file", lambda: "home/user/path")

    runner = CliRunner()
    result = runner.invoke(cli, ["admin", "user", "ls"])
    assert result.output == (
        f"[Errno 2] No such file or directory: 'home/user/path'\n"
        f"Configuration file not available.\n"
        f"Consider running 'obs --configure' to create one\n"
    )


def fake_exc_client():
    client = mock.Mock()
    client.user.list.return_value = fake_response()
    client.user.return_value = fake_response()
    client.qos.limits.return_value = fake_response()
    client.user.credentials.list.return_value = fake_response()

    return client


def fake_list_users(client, group_id, user_type="all", user_status="active", limit=""):
    user1 = {
        "userId": "jerrygarcia",
        "fullName": "Jerry Garcia",
        "emailAddr": "garcia@bgn.net",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "active": True,
    }
    user2 = {
        "userId": "johnthompson",
        "fullName": "John Thompson",
        "emailAddr": "",
        "address1": "",
        "city": "",
        "active": True,
    }
    return [user1, user2]


def test_ls(monkeypatch, client):
    monkeypatch.setattr(obs.libs.user, "list_user", fake_list_users)

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "admin",
            "user",
            "ls",
            "--group-id",
            "testing",
            "--user-type",
            "all",
            "--user-status",
            "active",
        ],
    )

    fixture_path = os.path.join("tests", "fixture", "tabulated_users")
    with io.open(fixture_path, "rt", encoding="utf8") as f:
        tabulated_users = f.read()

    assert result.output == tabulated_users


def test_except_ls(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_exc_client)

    runner = CliRunner()
    result = runner.invoke(cli, ["admin", "user", "ls"])

    assert result.output == (
        f"Users fetching failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_info(client, user_id, group_id):
    user = {
        "userId": "johnthompson",
        "fullName": "John Thompson",
        "emailAddr": "jgarc@geemail.com",
        "address1": "456 Shakedown St.",
        "city": "Portsmouth",
        "groupId": "testing",
        "canonicalUserId": "2c82bdc930155e8dc6860bfake",
        "active": True,
    }
    return user


def test_info(monkeypatch, client):
    monkeypatch.setattr(obs.libs.user, "info", fake_info)

    runner = CliRunner()
    result = runner.invoke(
        cli,
        ["admin", "user", "info", "--user-id", "StageTest", "--group-id", "testing"],
    )

    assert result.output == (
        f"ID: johnthompson\n"
        f"Name: John Thompson\n"
        f"Email: jgarc@geemail.com\n"
        f"Address: 456 Shakedown St.\n"
        f"City: Portsmouth\n"
        f"Group ID: testing\n"
        f"Canonical ID: 2c82bdc930155e8dc6860bfake\n"
        f"Active: True\n"
    )


def test_except_info(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_exc_client)

    runner = CliRunner()
    result = runner.invoke(
        cli,
        ["admin", "user", "info", "--user-id", "StageTest", "--group-id", "testing"],
    )

    assert result.output == (
        f"User info fetching failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_qos_info(client, user_id, group_id):
    limits = [
        {"type": "STORAGE_QUOTA_KBYTES", "value": -1},
        {"type": "REQUEST_RATE_LW", "value": -1},
        {"type": "REQUEST_RATE_LH", "value": -1},
        {"type": "DATAKBYTES_IN_LW", "value": -1},
        {"type": "DATAKBYTES_IN_LH", "value": -1},
        {"type": "DATAKBYTES_OUT_LW", "value": -1},
        {"type": "DATAKBYTES_OUT_LH", "value": -1},
        {"type": "STORAGE_QUOTA_COUNT", "value": -1},
    ]
    qos = {"groupId": "testing", "userId": "johnthompson", "qosLimitList": limits}
    return qos


def test_qos_info(monkeypatch, client):
    monkeypatch.setattr(obs.libs.qos, "info", fake_qos_info)
    monkeypatch.setattr(bitmath, "kB", fake_bitmath)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "qos", "info", "--user-id", "StageTest", "--group-id", "testing"]
    )

    assert result.output == (
        f"Group ID: testing\n" f"User ID: johnthompson\n" f"Storage Limit: Unlimited\n"
    )


def fake_bitmath(limits):
    limit = mock.Mock()
    limit.to_GiB.return_value.best_prefix.return_value = "0.49234 GiB"
    return limit


def test_except_qos_info(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_exc_client)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "qos", "info", "--user-id", "StageTest", "--group-id", "testing"]
    )

    assert result.output == (
        f"Storage limit fetching failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_qos():
    client = mock.Mock()
    client.qos.limits.return_value = "Done"
    return client


def test_qos_set(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_qos)

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "admin",
            "qos",
            "set",
            "--user-id",
            "user",
            "--group-id",
            "group",
            "--limit",
            100,
        ],
    )

    assert result.output == f"Storage limit changed\n"


def test_except_qos_set(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_exc_client)

    runner = CliRunner()
    result = runner.invoke(
        cli,
        [
            "admin",
            "qos",
            "set",
            "--user-id",
            "user",
            "--group-id",
            "group",
            "--limit",
            100,
        ],
    )

    assert (
        result.output == f"Storage limit change failed. \n"
        f"error [101]\n"
        f"URL: http://testing\n"
    )


def test_qos_rm(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_qos)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "qos", "rm", "--user-id", "user", "--group-id", "group"]
    )

    assert result.output == f"Storage limit removed\n"


def test_except_qos_rm(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_exc_client)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "qos", "rm", "--user-id", "user", "--group-id", "group"]
    )

    assert (
        result.output == f"Storage limit removal failed. \n"
        f"error [101]\n"
        f"URL: http://testing\n"
    )


def fake_cred_list(client, user_id, group_id):
    dt = datetime(2019, 9, 24, 13, 18, 7, 0).timestamp()
    credential = {
        "accessKey": "394b287c9efake",
        "secretKey": "IgP23gfnbrguu21YqFRw4+7Mfake",
        "createDate": dt,
        "active": True,
    }
    return [credential]


def fake_human_date(unixtime):
    return "1970-01-19 10:55:05+0700 (WIB)"


def test_cred_list(monkeypatch, client):
    monkeypatch.setattr(obs.libs.credential, "list", fake_cred_list)
    monkeypatch.setattr(obs.libs.utils, "human_date", fake_human_date)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "cred", "ls", "--user-id", "StageTest", "--group-id", "testing"]
    )

    assert result.output == (
        f"Access Key: 394b287c9efake\n"
        f"Secret Key: IgP23gfnbrguu21YqFRw4+7Mfake\n"
        f"Created: 1970-01-19 10:55:05+0700 (WIB)\n"
        f"Active: True\n"
    )


def test_except_cred_list(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_exc_client)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "cred", "ls", "--user-id", "StageTest", "--group-id", "testing"]
    )

    assert result.output == (
        f"Credentials listing failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_user():
    client = mock.Mock()
    client.user.return_value = fake_response()
    return client


def test_remove_user(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_user)
    monkeypatch.setattr(obs.libs.utils, "check", lambda response: None)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "user", "rm", "--user-id", "user", "--group-id", "group"]
    )

    assert result.output == f"User removed successfully\n"


def test_except_remove_user(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_user)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "user", "rm", "--user-id", "user", "--group-id", "group"]
    )
    assert result.output == (
        f"User removal failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_rm_creds():
    cred = mock.Mock()
    cred.user.credentials.return_value = fake_response()
    return cred


def test_remove_cred(monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_rm_creds)
    monkeypatch.setattr(obs.libs.utils, "check", lambda cred: None)

    runner = CliRunner()
    result = runner.invoke(cli, ["admin", "cred", "rm", "--access-key", "foo"])

    assert result.output == f"Credentials removed\n"


def test_except_remove_cred(client, monkeypatch):
    monkeypatch.setattr(admin_client, "get_admin_client", fake_rm_creds)

    runner = CliRunner()
    result = runner.invoke(cli, ["admin", "cred", "rm", "--access-key", "foo"])

    assert result.output == (
        f"Credential removal failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_status_creds():
    client = mock.Mock()
    client.user.credentials.status.return_value = fake_response()
    return client


def test_status_cred(monkeypatch):
    monkeypatch.setattr(obs.cli.admin.commands, "get_admin_client", fake_status_creds)
    monkeypatch.setattr(obs.libs.utils, "check", lambda cred: None)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "cred", "status", "--access-key", "Br432sd293fake"]
    )

    assert result.output == f"Credentials status changed\n"


def test_except_status_cred(client, monkeypatch):
    monkeypatch.setattr(obs.cli.admin.commands, "get_admin_client", fake_status_creds)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "cred", "status", "--access-key", "Br432sd293fake"]
    )
    assert result.output == (
        f"Credentials fetching failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )


def fake_create_creds():
    client = mock.Mock()
    client.user.credentials.return_value = fake_response()
    return client


def test_create_cred(monkeypatch):
    monkeypatch.setattr(obs.cli.admin.commands, "get_admin_client", fake_create_creds)
    monkeypatch.setattr(obs.libs.utils, "check", lambda cred: None)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "cred", "create", "--user-id", "user", "--group-id", "group"]
    )

    assert result.output == f"Credentials created\n"


def test_except_create_cred(client, monkeypatch):
    monkeypatch.setattr(obs.cli.admin.commands, "get_admin_client", fake_create_creds)

    runner = CliRunner()
    result = runner.invoke(
        cli, ["admin", "cred", "create", "--user-id", "user", "--group-id", "group"]
    )
    assert result.output == (
        f"Credential creation failed. \n" f"error [101]\n" f"URL: http://testing\n"
    )



import click

from obs.libs import config


def run_configure():
    """Prompt user interactively for config values and write those values to config_file."""
    cfg = config.Config()
    options = [
        (
            "user_access_key",
            "Access Key",
            "Access key and Secret key are your identifiers for object storage service",
        ),
        ("user_secret_key", "Secret Key"),
        (
            "user_url",
            "Object Storage Endpoint",
            f'Use "{cfg.user_url}" for Neo Object Storage.',
        ),
        (
            "user_gmt_policy",
            "Gmt Policy Path",
            f"Path to your gmt policy file, Leave as is 'notset' if you don't want to use Cloudian extension",
        ),
        (
            "admin_username",
            "Admin Username",
            f"""Admin username and password are your identifiers for your admin panel.""",
        ),
        ("admin_password", "Admin Password"),
        (
            "admin_url",
            "Admin URL",
            f"Admin url and port are your url and port location to your admin panel.",
        ),
        ("admin_port", "Admin port"),
        (
            "use_https",
            "Use HTTPS protocol",
            "All communication is protected when enabled, but it's slower than plain HTTP.",
        ),
        (
            "use_neo",
            "Use NEO compatibility",
            "Use NEO compatibility for adding specific policy id in Object Storage.",
        ),
    ]
    try:
        while True:
            click.secho("Put in new values or accept defaults.")
            click.secho("See user manual for complete description of options.")
            for option in options:
                prompt = option[1]
                val = getattr(cfg, option[0])
                if val not in (None, ""):
                    prompt += f" [{val}]"

                if len(option) >= 3:
                    click.secho(f"\n{option[2]}")

                val = input(prompt + ": ")
                # only set new value if user provide one
                if val != "":
                    setattr(cfg, option[0], val)

            val = input("\nSave settings? [y/N] ")
            if val.lower().startswith("y"):
                break

            val = input("Retry configuration? [Y/n] ")
            if val.lower().startswith("n"):
                raise EOFError()

        cfg.dump_config(options, cfg)

    except (EOFError, KeyboardInterrupt):
        click.secho(
            "\nConfiguration aborted. Changes were NOT saved.",
            fg="yellow",
            bold=True,
            err=True,
        )
        return



class UserProfile:
    """Accommodate user profile creation by setting and
    getting it's class variables."""

    active = True
    ldapEnabled = False
    userId = ""
    groupId = ""
    userType = "User"
    fullName = ""
    emailAddr = ""
    address1 = ""
    address2 = ""
    city = ""
    state = ""
    zip = ""
    country = ""
    phone = ""
    website = ""

    def dump(self, options, cfg):
        """Dump user profile data"""
        data = {}
        for option in options:
            value = getattr(cfg, option[0])
            data[option[0]] = value
        return data


def list_user(client, group_id, user_type="all", user_status="active", limit=""):
    """List all users."""
    users = client.user.list(
        groupId=group_id, userType=user_type, userStatus=user_status, limit=limit
    )
    return users


def info(client, user_id, group_id):
    """Get user info"""
    user = client.user(userId=user_id, groupId=group_id)
    return user


def create(client, data):
    """Create user"""
    response = client.user(method="PUT", json=data)
    return response


def update(client, data):
    """Update user info"""
    response = client.user(method="POST", json=data)
    return response


def remove(client, user_id, group_id):
    """Remove user"""
    response = client.user(method="DELETE", userId=user_id, groupId=group_id)
    return response



def info(client, user_id, group_id):
    """Get QoS info of specified user"""
    qos = client.qos.limits(method="GET", userId=user_id, groupId=group_id)
    return qos


def set(client, user_id, group_id, limit):
    """Set QoS of specified user"""
    qos = client.qos.limits(
        method="POST",
        userId=user_id,
        groupId=group_id,
        storageQuotaKBytes=limit,
        storageQuotaCount=-1,
        wlRequestRate=-1,
        hlRequestRate=-1,
        wlDataKBytesIn=-1,
        hlDataKBytesIn=-1,
        wlDataKBytesOut=-1,
        hlDataKBytesOut=-1,
    )
    return qos


def rm(client, user_id, group_id):
    """Remove QoS of specified user"""
    qos = client.qos.limits(method="DELETE", userId=user_id, groupId=group_id)
    return qos



import click

from obs.libs import user as user_lib


def prompt_user_profile():
    """Prompt user interactively for data"""
    user_profile = user_lib.UserProfile()
    options = [
        ("userId", "User ID"),
        ("groupId", "Group ID"),
        ("userType", "User Type"),
        ("fullName", "Full Name"),
        ("emailAddr", "Email"),
        ("address1", "Address"),
        ("city", "City"),
        ("state", "State"),
        ("zip", "Zip"),
        ("country", "Country"),
        ("phone", "Phone"),
        ("website", "Website"),
        ("active", "Active"),
        ("ldapEnabled", "LDAP Enabled"),
    ]
    try:
        while True:
            click.secho("Put in new values or accept defaults.")
            for option in options:
                prompt = option[1]
                val = getattr(user_profile, option[0])
                if val not in (None, ""):
                    prompt += f" [{val}]"

                if len(option) >= 3:
                    click.secho(f"\n{option[2]}")

                val = input(prompt + ": ")
                # only set new value if user provide one
                if val != "":
                    setattr(user_profile, option[0], val)

            val = input("\nSave settings? [y/N] ")
            if val.lower().startswith("y"):
                break

            val = input("Retry entry? [Y/n] ")
            if val.lower().startswith("n"):
                raise EOFError()

        data = user_profile.dump(options, user_profile)
        return data

    except (EOFError, KeyboardInterrupt):
        click.secho("\nData enty aborted.", fg="yellow", bold=True, err=True)
        return



import datetime


def usage(client, user_id, group_id):
    """Get storage usage."""
    # for `raw` granularity, the interval between
    # "startTime" and "endTime" must not exceed 24 hours.
    now = datetime.datetime.now()
    yesterday = now - datetime.timedelta(1)
    du = client.usage(
        id=f"{group_id}|{user_id}",
        operation="SB",
        startTime=f"{yesterday:%Y%m%d%H%M}",
        endTime=f"{now:%Y%m%d%H%M}",
        granularity="raw",
        reversed="true",
    )
    return du



import os
import sys
import logging

from flask import Flask
from flask_cors import CORS

sys.path.append(os.path.abspath("../../"))
from . import configs
from .controllers import api_blueprint


def create_app():

    app = Flask(__name__)
    app.config.from_object(configs.Config)

    CORS(app, resources={r"/api/*": {"origins": "*"}})

    app.register_blueprint(api_blueprint)
    logging.basicConfig(
        format="[%(asctime)s] [%(process)d] [%(levelname)s] %(filename)s - %(funcName)s: %(message)s"
    )

    return app



import tabulate
import click

from obs.libs import user as user_lib
from obs.cli.admin import user_profile
from obs.libs import utils


def list_user(client, group_id, user_type, user_status, limit):
    try:
        users = user_lib.list_user(
            client=client,
            group_id=group_id,
            user_type=user_type,
            user_status=user_status,
            limit=limit,
        )
        utils.check(users)
        number = 1  # must start from 1 (as user data)
        results = []
        for user in users:
            data = {
                "No": number,
                "User": user["userId"],
                "Name": user["fullName"],
                "Email": user["emailAddr"],
                "Address": user["address1"],
                "City": user["city"],
                "Status": user["active"],
            }
            number += 1
            results.append(data)
        click.secho(tabulate.tabulate(results, headers="keys", tablefmt="grid"))
    except Exception as exc:
        click.secho(f"Users fetching failed. \n{exc}", fg="yellow", bold=True, err=True)


def info(client, user_id, group_id):
    try:
        user = user_lib.info(client=client, user_id=user_id, group_id=group_id)
        utils.check(user)
        result = (
            f"ID: {user['userId']}\n"
            f"Name: {user['fullName']}\n"
            f"Email: {user['emailAddr']}\n"
            f"Address: {user['address1']}\n"
            f"City: {user['city']}\n"
            f"Group ID: {user['groupId']}\n"
            f"Canonical ID: {user['canonicalUserId']}\n"
            f"Active: {user['active']}"
        )
        click.secho(result)
    except Exception as exc:
        click.secho(
            f"User info fetching failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def create(client):
    try:
        data = user_profile.prompt_user_profile()
        response = user_lib.create(client, utils.user_sanitize(data))
        utils.check(response)
        click.secho(f"User created successfully", fg="green")
    except Exception as exc:
        click.secho(f"User creation failed. \n{exc}", fg="yellow", bold=True, err=True)


def suspend(client, user_id, group_id, status):
    try:
        user_info = user_lib.info(client=client, user_id=user_id, group_id=group_id)
        message = "suspended" if status is True else "unsuspended"
        data = {key: val for key, val in user_info.items() if "canonical" not in key}
        data["active"] = not status
        response = user_lib.update(client, data)
        utils.check(response)
        click.secho(f"User {message} successfully", fg="green")
    except Exception as exc:
        click.secho(
            f"User suspension failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def remove(client, user_id, group_id):
    try:
        response = user_lib.remove(client=client, user_id=user_id, group_id=group_id)
        utils.check(response)
        click.secho(f"User removed successfully", fg="green")
    except Exception as exc:
        click.secho(f"User removal failed. \n{exc}", fg="yellow", bold=True, err=True)
